@@ -1,11 +1,7 @@
 // libraries for User interface:
 
-
 #include <tcl.h>
 #include <tk.h>
-// #include "/opt/tcltk86/include/tcl.h"
-// #include "/opt/tcltk86/include/tk.h"
-
 
 #if TK_MAJOR_VERSION==8 && TK_MINOR_VERSION>=4
 #define tcl_const const
@@ -13,6 +9,8 @@
 #define tcl_const
 #endif
 
+#define GL_GLEXT_PROTOTYPES
+
 
 #  if defined(TOGL_AGL) || defined(TOGL_AGL_CLASSIC)
 #    include <OpenGL/gl.h>
@@ -38,3 +36,5 @@
 #define GL_CLAMP_TO_EDGE 0x812F
 #endif
 
+
+
