@@ -20,19 +20,21 @@
 // in the configure/make phases, with the 
 // right version number
 #ifdef WIN32
-#define PACKAGE_VERSION "4.9.13"
+#define PACKAGE_VERSION "5.1"
 #endif
 
 
 #ifdef WIN32
-    #define DLL_HEADER   __declspec(dllexport)
+   #if NGINTERFACE_EXPORTS || NGLIB_EXPORTS || nglib_EXPORTS
+      #define DLL_HEADER   __declspec(dllexport)
+   #else
+      #define DLL_HEADER   __declspec(dllimport)
+   #endif
 #else
-    #define DLL_HEADER 
+   #define DLL_HEADER 
 #endif
 
 
-
-
 #define noDEMOVERSION
 #define noDEVELOP
 #define noSTEP
