@@ -1,7 +1,7 @@
 #ifndef FILE_MYSTDLIB
 #define FILE_MYSTDLIB
 
-#ifndef WIN32
+#ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
@@ -27,10 +27,11 @@
 #include <typeinfo>
 
 #ifdef PARALLEL
-#undef SEEK_SET
-#undef SEEK_CUR
-#undef SEEK_END
+// #undef SEEK_SET
+// #undef SEEK_CUR
+// #undef SEEK_END
 #include <mpi.h>
+#include <unistd.h>  // for usleep (only for parallel)
 #endif
 
 #ifdef _OPENMP
