@@ -17,7 +17,7 @@
 */
 
 #ifdef WIN32
-   #ifdef NGINTERFACE_EXPORTS
+   #if NGINTERFACE_EXPORTS || NGLIB_EXPORTS || nglib_EXPORTS
       #define DLL_HEADER   __declspec(dllexport)
    #else
       #define DLL_HEADER   __declspec(dllimport)
@@ -37,6 +37,7 @@
 
 // implemented element types:
 enum NG_ELEMENT_TYPE { 
+  NG_PNT = 0,
   NG_SEGM = 1, NG_SEGM3 = 2,
   NG_TRIG = 10, NG_QUAD=11, NG_TRIG6 = 12, NG_QUAD6 = 13,
   NG_TET = 20, NG_TET10 = 21, 
@@ -215,6 +216,8 @@ extern "C" {
   DLL_HEADER const NG_EDGE * Ng_ME_GetEdges (NG_ELEMENT_TYPE et);
   DLL_HEADER const NG_FACE * Ng_ME_GetFaces (NG_ELEMENT_TYPE et);
 
+  DLL_HEADER void Ng_UpdateTopology();
+
   DLL_HEADER int Ng_GetNEdges();
   DLL_HEADER int Ng_GetNFaces();
 
@@ -270,33 +273,22 @@ extern "C" {
 					double * p3d, double * jacobian);
 
 #ifdef PARALLEL
-  // Is Element ei an element of this processor ??
-  bool Ng_IsGhostEl (int ei);
 
-  void Ng_SetGhostEl(const int ei, const bool aisghost );
+  // the folling functions are 0-base  !!
 
-  bool Ng_IsGhostSEl (int ei);
-
-  void Ng_SetGhostSEl(const int ei, const bool aisghost );
-
-  bool Ng_IsGhostVert ( int pnum );
-  bool Ng_IsGhostEdge ( int ednum );
-  bool Ng_IsGhostFace ( int fanum );
-
-  bool Ng_IsExchangeEl ( int elnum );
-  bool Ng_IsExchangeSEl ( int selnr );
-
-  void Ng_UpdateOverlap ();
-  int Ng_Overlap();
-/*   void Ng_SetGhostVert ( const int pnum, const bool aisghost ); */
-/*   void Ng_SetGhostEdge ( const int ednum, const bool aisghost ); */
-/*   void Ng_SetGhostFace ( const int fanum, const bool aisghost ); */
+  // number on distant processor 
+  // returns pairs  (dist_proc, num_on_dist_proc)
+  int NgPar_GetDistantNodeNums ( int nodetype, int locnum, int * pnums );
+  int NgPar_GetNDistantNodeNums ( int nodetype, int locnum );
+  
+  int NgPar_GetGlobalNodeNum (int nodetype, int locnum);
 
 #endif
   
   namespace netgen {
   // #include "../visualization/soldata.hpp"
     class SolutionData;
+    class MouseEventHandler;
   }
 
   enum Ng_SolutionType
@@ -332,6 +324,8 @@ extern "C" {
   DLL_HEADER void Ng_ClearSolutionData();
   // redraw 
   DLL_HEADER void Ng_Redraw();
+  ///
+  DLL_HEADER void Ng_SetMouseEventHandler (netgen::MouseEventHandler * handler);
   //
   DLL_HEADER void Ng_SetVisualizationParameter (const char * name, 
                                                 const char * value);
@@ -391,6 +385,13 @@ extern "C" {
   // if qualityloss is not equal to NULL at input, a (1-based) list of qualitylosses (due to projection)
   // is saved in *qualityloss, its size is the return value
   DLL_HEADER int Ng_Bisect_WithInfo ( const char * refinementfile, double ** qualityloss);
+
+  typedef void * Ng_Mesh;
+  DLL_HEADER Ng_Mesh Ng_SelectMesh (Ng_Mesh mesh);
+
+  DLL_HEADER void Ng_GetArgs (int & argc, char ** &argv);
+
+
 #ifdef __cplusplus
 }
 #endif
