@@ -32,10 +32,12 @@ namespace netgen
       "FEAP Format", ".mesh",
       "Elmer Format", "*",
       "STL Format", ".stl",
+      "STL Extended Format", ".stl",
       "VRML Format", ".*",
       "Gmsh Format", ".gmsh",
       "Gmsh2 Format", ".gmsh2",
       "OpenFOAM 1.5+ Format", "*",
+	  "OpenFOAM 1.5+ Compressed", "*",
       "JCMwave Format", ".jcm",
       "TET Format", ".tet",
       //      { "Chemnitz Format" },
@@ -53,9 +55,11 @@ namespace netgen
 
 bool WriteUserFormat (const string & format,
 		      const Mesh & mesh,
-		      const CSGeometry & geom,
+		      const NetgenGeometry & hgeom,
 		      const string & filename)
 {
+  const CSGeometry & geom = *dynamic_cast<const CSGeometry*> (&hgeom);
+
   PrintMessage (1, "Export mesh to file ", filename,
 		", format is ", format);
 
@@ -93,6 +97,13 @@ bool WriteUserFormat (const string & format,
   else if (format == "STL Format")
     WriteSTLFormat (mesh, filename);
 
+  // Philippose - 16 August 2010
+  // Added additional STL Export in which
+  // each face of the geometry is treated
+  // as a separate "solid" entity
+  else if (format == "STL Extended Format")
+	WriteSTLExtFormat (mesh, filename);
+
   else if (format == "VRML Format")
     WriteVRMLFormat (mesh, 1, filename);
 
@@ -116,7 +127,10 @@ bool WriteUserFormat (const string & format,
   // Philippose - 25/10/2009
   // Added OpenFOAM 1.5+ Mesh export capability
   else if (format == "OpenFOAM 1.5+ Format")
-    WriteOpenFOAM15xFormat (mesh, filename);
+    WriteOpenFOAM15xFormat (mesh, filename, false);
+
+  else if (format == "OpenFOAM 1.5+ Compressed")
+    WriteOpenFOAM15xFormat (mesh, filename, true);
 
   else if (format == "JCMwave Format")
     WriteJCMFormat (mesh, geom, filename);
@@ -294,17 +308,22 @@ void WriteSTLFormat (const Mesh & mesh,
 {
   cout << "\nWrite STL Surface Mesh" << endl;
 
-  ofstream outfile (filename.c_str());
+  ostream *outfile;
+
+  if(filename.substr(filename.length()-3,3) == ".gz")
+	  outfile = new ogzstream(filename.c_str());
+  else
+	  outfile = new ofstream(filename.c_str());
 
   int i;
 
-  outfile.precision(10);
+  outfile->precision(10);
 
-  outfile << "solid" << endl;
+  *outfile << "solid" << endl;
 
   for (i = 1; i <= mesh.GetNSE(); i++)
     {
-      outfile << "facet normal ";
+      *outfile << "facet normal ";
       const Point3d& p1 = mesh.Point(mesh.SurfaceElement(i).PNum(1));
       const Point3d& p2 = mesh.Point(mesh.SurfaceElement(i).PNum(2));
       const Point3d& p3 = mesh.Point(mesh.SurfaceElement(i).PNum(3));
@@ -315,23 +334,117 @@ void WriteSTLFormat (const Mesh & mesh,
 	  normal /= (normal.Length());
 	}
 
-      outfile << normal.X() << " " << normal.Y() << " " << normal.Z() << "\n";
-      outfile << "outer loop\n";
+      *outfile << normal.X() << " " << normal.Y() << " " << normal.Z() << "\n";
+      *outfile << "outer loop\n";
 
-      outfile << "vertex " << p1.X() << " " << p1.Y() << " " << p1.Z() << "\n";
-      outfile << "vertex " << p2.X() << " " << p2.Y() << " " << p2.Z() << "\n";
-      outfile << "vertex " << p3.X() << " " << p3.Y() << " " << p3.Z() << "\n";
+      *outfile << "vertex " << p1.X() << " " << p1.Y() << " " << p1.Z() << "\n";
+      *outfile << "vertex " << p2.X() << " " << p2.Y() << " " << p2.Z() << "\n";
+      *outfile << "vertex " << p3.X() << " " << p3.Y() << " " << p3.Z() << "\n";
 
-      outfile << "endloop\n";
-      outfile << "endfacet\n";
+      *outfile << "endloop\n";
+      *outfile << "endfacet\n";
     }
-  outfile << "endsolid" << endl;
+  *outfile << "endsolid" << endl;
 }
 
 
 
 
 
+/*
+ *  Philippose - 16 August 2010
+ *  Save surface mesh as STL file
+ *  with a separate solid definition
+ *  for each face
+ *  - This helps in splitting up the
+ *    STL into named boundary faces
+ *    when using a third-party mesher
+ */
+void WriteSTLExtFormat (const Mesh & mesh,
+		     const string & filename)
+{
+  cout << "\nWrite STL Surface Mesh (with separated boundary faces)" << endl;
+
+  ostream *outfile;
+
+  if(filename.substr(filename.length()-3,3) == ".gz")
+	  outfile = new ogzstream(filename.c_str());
+  else
+	  outfile = new ofstream(filename.c_str());
+
+  outfile->precision(10);
+
+  int numBCs = 0;
+
+  Array<int> faceBCs;
+  TABLE<int> faceBCMapping;
+
+  faceBCs.SetSize(mesh.GetNFD());
+  faceBCMapping.SetSize(mesh.GetNFD());
+
+  faceBCs = -1;
+
+  // Collect the BC numbers used in the mesh
+  for(int faceNr = 1; faceNr <= mesh.GetNFD(); faceNr++)
+  {
+	  int bcNum = mesh.GetFaceDescriptor(faceNr).BCProperty();
+
+	  if(faceBCs.Pos(bcNum) < 0)
+	  {
+        numBCs++;
+		  faceBCs.Set(numBCs,bcNum);
+        faceBCMapping.Add1(numBCs,faceNr);        
+	  }
+     else
+     {
+        faceBCMapping.Add1(faceBCs.Pos(bcNum)+1,faceNr);
+     }
+  }
+
+  faceBCs.SetSize(numBCs);
+  faceBCMapping.ChangeSize(numBCs);
+
+  // Now actually write the data to file
+  for(int bcInd = 1; bcInd <= faceBCs.Size(); bcInd++)
+  {
+      *outfile << "solid Boundary_" << faceBCs.Elem(bcInd) << "\n";
+
+      for(int faceNr = 1;faceNr <= faceBCMapping.EntrySize(bcInd); faceNr++)
+      {
+          Array<SurfaceElementIndex> faceSei;
+          mesh.GetSurfaceElementsOfFace(faceBCMapping.Get(bcInd,faceNr),faceSei);
+
+          for (int i = 0; i < faceSei.Size(); i++)
+          {
+        	  *outfile << "facet normal ";
+        	  const Point3d& p1 = mesh.Point(mesh.SurfaceElement(faceSei[i]).PNum(1));
+        	  const Point3d& p2 = mesh.Point(mesh.SurfaceElement(faceSei[i]).PNum(2));
+        	  const Point3d& p3 = mesh.Point(mesh.SurfaceElement(faceSei[i]).PNum(3));
+
+        	  Vec3d normal = Cross(p2-p1,p3-p1);
+        	  if (normal.Length() != 0)
+        	  {
+        		  normal /= (normal.Length());
+        	  }
+
+        	  *outfile << normal.X() << " " << normal.Y() << " " << normal.Z() << "\n";
+        	  *outfile << "outer loop\n";
+
+        	  *outfile << "vertex " << p1.X() << " " << p1.Y() << " " << p1.Z() << "\n";
+        	  *outfile << "vertex " << p2.X() << " " << p2.Y() << " " << p2.Z() << "\n";
+        	  *outfile << "vertex " << p3.X() << " " << p3.Y() << " " << p3.Z() << "\n";
+
+        	  *outfile << "endloop\n";
+        	  *outfile << "endfacet\n";
+          }
+      }
+      *outfile << "endsolid Boundary_" << faceBCs.Elem(bcInd) << "\n";
+  }
+}
+
+
+
+
 /*
  *
  *  write surface mesh as VRML file
