@@ -86,8 +86,7 @@ AdFront3 :: ~AdFront3 ()
 
 void AdFront3 :: GetPoints (Array<Point<3> > & apoints) const
 {
-  for (PointIndex pi = PointIndex::BASE; 
-       pi < points.Size()+PointIndex::BASE; pi++)
+  for (PointIndex pi = points.Begin(); pi < points.End(); pi++)
     
     apoints.Append (points[pi].P());
 }
@@ -106,7 +105,8 @@ PointIndex AdFront3 :: AddPoint (const Point<3> & p, PointIndex globind)
   else
     {
       points.Append (FrontPoint3 (p, globind));
-      return points.Size()-1+PointIndex::BASE;
+      return --points.End();
+      // return points.Size()-1+PointIndex::BASE;
     }
 }
 
@@ -301,9 +301,8 @@ void AdFront3 :: RebuildInternalTables ()
 
   int np = points.Size();
 
-  for (int i = PointIndex::BASE; 
-       i < np+PointIndex::BASE; i++)
-    points[i].cluster = i;
+  for (PointIndex pi = points.Begin(); pi < points.End(); pi++)
+    points[pi].cluster = pi;
   
   NgProfiler::StopTimer (timer_a);	  
   NgProfiler::StartTimer (timer_b);	  
@@ -400,9 +399,8 @@ void AdFront3 :: RebuildInternalTables ()
     {
       for (int i = 1; i <= faces.Size(); i++)
 	faces.Elem(i).cluster = 1;
-      for (int i = PointIndex::BASE; 
-	   i < points.Size()+PointIndex::BASE; i++)
-	points[i].cluster = 1;
+      for (PointIndex pi = points.Begin(); pi < points.End(); pi++)
+	points[pi].cluster = 1;
     }
 
   if (hashon) 
@@ -508,14 +506,13 @@ int AdFront3 :: GetLocals (int fstind,
 
   INDEX i, j;
   PointIndex pstind;
-  INDEX pi;
   Point3d midp, p0;
 
-  static Array<int, PointIndex::BASE> invpindex;
+  //  static Array<int, PointIndex::BASE> invpindex;
   
-  static Array<MiniElement2d> locfaces2;           //all local faces in radius xh
-  static Array<int> locfaces3;           // all faces in outer radius relh
-  static Array<INDEX> findex2;
+  Array<MiniElement2d> locfaces2;           //all local faces in radius xh
+  Array<int> locfaces3;           // all faces in outer radius relh
+  Array<INDEX> findex2;
 
   locfaces2.SetSize(0);
   locfaces3.SetSize(0);
@@ -591,7 +588,7 @@ int AdFront3 :: GetLocals (int fstind,
   for (i = 1; i <= locfaces.Size(); i++)
     for (j = 1; j <= locfaces.Get(i).GetNP(); j++)
       {
-	pi = locfaces.Get(i).PNum(j);
+	PointIndex pi = locfaces.Get(i).PNum(j);
 	invpindex[pi] = -1;
       }
 
@@ -599,7 +596,7 @@ int AdFront3 :: GetLocals (int fstind,
     {
       for (j = 1; j <= locfaces.Get(i).GetNP(); j++)
 	{
-	  pi = locfaces.Get(i).PNum(j);
+	  PointIndex pi = locfaces.Get(i).PNum(j);
 	  if (invpindex[pi] == -1)
 	    {
 	      pindex.Append (pi);
@@ -661,9 +658,9 @@ void AdFront3 :: GetGroup (int fi,
 			   Array<MeshPoint> & grouppoints,
 			   Array<MiniElement2d> & groupelements,
 			   Array<PointIndex> & pindex,
-			   Array<INDEX> & findex) const
+			   Array<INDEX> & findex) 
 {
-  static Array<char> pingroup;
+  // static Array<char> pingroup;
   int i, j, changed;
 
   pingroup.SetSize(points.Size());
@@ -699,16 +696,16 @@ void AdFront3 :: GetGroup (int fi,
   while (changed);
 
 
-  static Array<int> invpindex;
+  //  static Array<int> invpindex;
   invpindex.SetSize (points.Size());
   
 
-  for (i = 1; i <= points.Size(); i++)
+  for (PointIndex pi = points.Begin(); pi < points.End(); pi++)
     if (points.Get(i).Valid())
       {
-	grouppoints.Append (points.Get(i).P());
-	pindex.Append (i);
-	invpindex.Elem(i) = pindex.Size();
+	grouppoints.Append (points[pi].P());
+	pindex.Append (pi);
+	invpindex[pi] = pindex.Size();
       }
 
   for (i = 1; i <= faces.Size(); i++)
@@ -772,7 +769,7 @@ void AdFront3 :: SetStartFront (int /* baseelnp */)
 
 bool AdFront3 :: Inside (const Point<3> & p) const
 {
-  int i, cnt;
+  int cnt;
   Vec3d n, v1, v2;
   DenseMatrix a(3), ainv(3);
   Vector b(3), u(3);
@@ -783,7 +780,7 @@ bool AdFront3 :: Inside (const Point<3> & p) const
   n.Z() = -0.43989;
 
   cnt = 0;
-  for (i = 1; i <= faces.Size(); i++)
+  for (int i = 1; i <= faces.Size(); i++)
     if (faces.Get(i).Valid())
       {
 	const Point<3> & p1 = points[faces.Get(i).Face().PNum(1)].P();
@@ -827,36 +824,29 @@ bool AdFront3 :: Inside (const Point<3> & p) const
 int AdFront3 :: SameSide (const Point<3> & lp1, const Point<3> & lp2,
 			  const Array<int> * testfaces) const
 {
-  int i, ii, cnt;
-
   const Point<3> *line[2];
   line[0] = &lp1;
   line[1] = &lp2;
 
 
-  cnt = 0;
-
   Point3d pmin(lp1);
   Point3d pmax(lp1);
   pmin.SetToMin (lp2);
   pmax.SetToMax (lp2);
-
-  static Array<int> aprif;
+  
+  ArrayMem<int, 100> aprif;
   aprif.SetSize(0);
   
   if (!testfaces)
     facetree->GetIntersecting (pmin, pmax, aprif);
   else
-    {
-      for (i = 1; i <= testfaces->Size(); i++)
-	aprif.Append (testfaces->Get(i));
-    }
+    for (int i = 1; i <= testfaces->Size(); i++)
+      aprif.Append (testfaces->Get(i));
 
-  //  (*testout) << "test ss, p1,p2 = " << lp1 << lp2 << ", inters = " << aprif.Size() << endl;
-  //  for (i = 1; i <= faces.Size(); i++)
-  for (ii = 1; ii <= aprif.Size(); ii++)
+  int cnt = 0;
+  for (int ii = 1; ii <= aprif.Size(); ii++)
     {
-      i = aprif.Get(ii);
+      int i = aprif.Get(ii);
       
       if (faces.Get(i).Valid())
 	{
@@ -867,7 +857,6 @@ int AdFront3 :: SameSide (const Point<3> & lp1, const Point<3> & lp2,
 	  	  
 	  if (IntersectTriangleLine (&tri[0], &line[0]))
 	    cnt++;
-	  
 	}
     }
 
