@@ -17,20 +17,20 @@ namespace netgen
   ostream * myerr = &cerr;
 
 
-  // Flags globflags; // not used anymoure
-  Flags parameters;
+  //  Flags parameters;
 
 
   int silentflag = 0;
   int testmode = 0;
 
-  MeshingParameters mparam;
   volatile multithreadt multithread;
 
   string ngdir = ".";
 
   Array<int> tets_in_qualclass;
 
+  int h_argc = 0;
+  char ** h_argv = NULL;
 
   multithreadt :: multithreadt()
   {
