@@ -4,14 +4,6 @@
 
 namespace netgen
 {
-
-
-  ostream & operator<<(ostream  & s, const MeshPoint & pt)
-  {
-    s << Point<3> (pt);
-    return s;
-  }
-
   int MultiPointGeomInfo :: 
   AddPointGeomInfo (const PointGeomInfo & gi)
   {
@@ -30,6 +22,40 @@ namespace netgen
   }
   
 
+
+#ifdef PARALLEL
+  MPI_Datatype MeshPoint :: MyGetMPIType ( )
+  { 
+    static MPI_Datatype type = NULL;
+    static MPI_Datatype htype = NULL;
+    if (!type)
+      {
+	MeshPoint hp;
+	int blocklen[] = { 3, 1, 1 };
+	MPI_Aint displ[] = { (char*)&hp.x[0] - (char*)&hp,
+			     (char*)&hp.layer - (char*)&hp,
+			     (char*)&hp.singular - (char*)&hp };
+	MPI_Datatype types[] = { MPI_DOUBLE, MPI_INT, MPI_DOUBLE };
+	*testout << "displ = " << displ[0] << ", " << displ[1] << ", " << displ[2] << endl;
+	*testout << "sizeof = " << sizeof (MeshPoint) << endl;
+	MPI_Type_create_struct (3, blocklen, displ, types, &htype);
+	MPI_Type_commit ( &htype );
+	MPI_Aint lb, ext;
+	MPI_Type_get_extent (htype, &lb, &ext);
+	*testout << "lb = " << lb << endl;
+	*testout << "ext = " << ext << endl;
+	ext = sizeof (MeshPoint);
+	MPI_Type_create_resized (htype, lb, ext, &type);
+	MPI_Type_commit ( &type );
+	
+      }
+    return type;
+  }
+#endif
+
+
+
+
   Segment :: Segment() 
   {
     pnums[0] = -1;
@@ -141,13 +167,11 @@ namespace netgen
     index = 0;
     badel = 0;
     deleted = 0;
+    visible = 1;
     typ = TRIG;
     orderx = ordery = 1;
     refflag = 1;
     strongrefflag = false;
-#ifdef PARALLEL
-    isghost = 0;
-#endif
   } 
 
 
@@ -162,6 +186,7 @@ namespace netgen
     index = 0;
     badel = 0;
     deleted = 0;
+    visible = 1;
     switch (np)
       {
       case 3: typ = TRIG; break;
@@ -172,9 +197,6 @@ namespace netgen
     orderx = ordery = 1;
     refflag = 1;
     strongrefflag = false;
-#ifdef PARALLEL
-    isghost = 0;
-#endif
   } 
 
   Element2d :: Element2d (ELEMENT_TYPE atyp)
@@ -190,13 +212,10 @@ namespace netgen
     index = 0;
     badel = 0;
     deleted = 0;
+    visible = 1;
     orderx = ordery = 1;
     refflag = 1;
     strongrefflag = false;
-#ifdef PARALLEL
-    isghost = 0;
-#endif
-
   } 
 
 
@@ -219,12 +238,8 @@ namespace netgen
     refflag = 1;
     strongrefflag = false;
     deleted = 0;
+    visible = 1;
     orderx = ordery = 1;
-
-#ifdef PARALLEL
-    isghost = 0;
-#endif
-
   }
 
   Element2d :: Element2d (int pi1, int pi2, int pi3, int pi4)
@@ -246,11 +261,8 @@ namespace netgen
     refflag = 1;
     strongrefflag = false;
     deleted = 0;
+    visible = 1;
     orderx = ordery = 1;
-
-#ifdef PARALLEL
-    isghost = 0;
-#endif
   }
 
 
@@ -297,6 +309,12 @@ namespace netgen
           Swap (pnum[1], pnum[2]);
           break;
         }
+      case TRIG6:
+        {
+          Swap (pnum[1], pnum[2]);
+          Swap (pnum[4], pnum[5]);
+          break;
+        }
       case QUAD:
         {
           Swap (pnum[0], pnum[3]);
@@ -415,7 +433,7 @@ namespace netgen
                      DenseMatrix & trans) const
   {
     int np = GetNP();
-    static DenseMatrix pmat(2, np), dshape(2, np);
+    DenseMatrix pmat(2, np), dshape(2, np);
     pmat.SetSize (2, np);
     dshape.SetSize (2, np);
 
@@ -464,7 +482,6 @@ namespace netgen
   }
 
 
-
   void Element2d :: GetShape (const Point2d & p, Vector & shape) const
   {
     if (shape.Size() != GetNP())
@@ -563,7 +580,6 @@ namespace netgen
 
 
 
-
   void Element2d :: 
   GetDShapeNew (const Point<2> & p, MatrixFixWidth<2> & dshape) const
   {
@@ -630,8 +646,8 @@ namespace netgen
   {
     int i, j;
     int nip = GetNIP();
-    static DenseMatrix trans(2,2);
-    static DenseMatrix pmat;
+    DenseMatrix trans(2,2);
+    DenseMatrix pmat;
   
     pmat.SetSize (2, GetNP());
     GetPointMatrix (points, pmat);
@@ -748,8 +764,8 @@ namespace netgen
       }
 
     int nip = GetNIP();
-    static DenseMatrix trans(2,2), dtrans(2,2);
-    static DenseMatrix pmat, vmat;
+    DenseMatrix trans(2,2), dtrans(2,2);
+    DenseMatrix pmat, vmat;
   
     pmat.SetSize (2, GetNP());
     vmat.SetSize (2, GetNP());
@@ -811,8 +827,8 @@ namespace netgen
   {
     int i, j;
     int nip = GetNIP();
-    static DenseMatrix trans(2,2);
-    static DenseMatrix pmat;
+    DenseMatrix trans(2,2);
+    DenseMatrix pmat;
   
     pmat.SetSize (2, GetNP());
 
@@ -852,6 +868,7 @@ namespace netgen
 
 
 
+
   void Element2d :: ComputeIntegrationPointData () const
   {
     switch (np)
@@ -891,7 +908,6 @@ namespace netgen
 
 
 
-
   ostream & operator<<(ostream  & s, const Element2d & el)
   {
     s << "np = " << el.GetNP();
@@ -931,7 +947,6 @@ namespace netgen
 
 #ifdef PARALLEL
     partitionNumber = -1;
-    isghost = 0;
 #endif
 
   }
@@ -965,10 +980,6 @@ namespace netgen
       default: cerr << "Element::Element: unknown element with " << np << " points" << endl;
       }
     orderx = ordery = orderz = 1;
-
-#ifdef PARALLEL
-    isghost = 0;
-#endif
   }
 
   void Element :: SetOrder (const int aorder) 
@@ -1006,10 +1017,6 @@ namespace netgen
     flags.deleted = 0;
     flags.fixed = 0;
     orderx = ordery = orderz = 1;
-
-#ifdef PARALLEL
-    isghost = 0;
-#endif
   }
 
 
@@ -1500,6 +1507,7 @@ namespace netgen
         {
           cout << "GetNodesLocal not impelemented for element " << GetType() << endl;
           np = 0;
+	  pp = NULL;
         }
       }
   
@@ -1692,7 +1700,7 @@ namespace netgen
                      DenseMatrix & trans) const
   {
     int np = GetNP();
-    static DenseMatrix pmat(3, np), dshape(3, np);
+    DenseMatrix pmat(3, np), dshape(3, np);
     pmat.SetSize (3, np);
     dshape.SetSize (3, np);
 
@@ -1739,7 +1747,6 @@ namespace netgen
   }
 
 
-
   void Element :: GetShape (const Point<3> & hp, Vector & shape) const
   {
     Point3d p = hp;
@@ -1896,7 +1903,6 @@ namespace netgen
 
 
 
-
   void Element :: 
   GetDShape (const Point<3> & hp, DenseMatrix & dshape) const
   {
@@ -1926,7 +1932,6 @@ namespace netgen
   }
 
 
-
   void Element :: 
   GetDShapeNew (const Point<3> & p, MatrixFixWidth<3> & dshape) const
   {
@@ -1990,13 +1995,6 @@ namespace netgen
                   DenseMatrix & pmat) const
   {
     int np = GetNP();
-    /*
-      if (pmat.Width() != np || pmat.Height() != 3)
-      {
-      cerr << "Element::GetPointMatrix: sizes don't fit" << endl;
-      return;
-      }
-    */
     for (int i = 1; i <= np; i++)
       {
         const Point3d & p = points.Get(PNum(i));
@@ -2009,26 +2007,23 @@ namespace netgen
 
 
 
-
-
   double Element :: CalcJacobianBadness (const T_POINTS & points) const
   {
-    int i, j;
     int nip = GetNIP();
-    static DenseMatrix trans(3,3);
-    static DenseMatrix pmat;
+    DenseMatrix trans(3,3);
+    DenseMatrix pmat;
   
     pmat.SetSize (3, GetNP());
     GetPointMatrix (points, pmat);
 
     double err = 0;
-    for (i = 1; i <= nip; i++)
+    for (int i = 1; i <= nip; i++)
       {
         GetTransformation (i, pmat, trans);
 
         // Frobenius norm
         double frob = 0;
-        for (j = 1; j <= 9; j++)
+        for (int j = 1; j <= 9; j++)
           frob += sqr (trans.Get(j));
         frob = sqrt (frob);
         frob /= 3;
@@ -2051,8 +2046,8 @@ namespace netgen
   {
     int i, j, k;
     int nip = GetNIP();
-    static DenseMatrix trans(3,3), dtrans(3,3), hmat(3,3);
-    static DenseMatrix pmat, vmat;
+    DenseMatrix trans(3,3), dtrans(3,3), hmat(3,3);
+    DenseMatrix pmat, vmat;
   
     pmat.SetSize (3, GetNP());
     vmat.SetSize (3, GetNP());
@@ -2126,8 +2121,8 @@ namespace netgen
                                int pi, Vec<3> & grad) const
   {
     int nip = GetNIP();
-    static DenseMatrix trans(3,3), dtrans(3,3), hmat(3,3);
-    static DenseMatrix pmat, vmat;
+    DenseMatrix trans(3,3), dtrans(3,3), hmat(3,3);
+    DenseMatrix pmat, vmat;
   
     pmat.SetSize (3, GetNP());
     vmat.SetSize (3, GetNP());
@@ -2209,7 +2204,6 @@ namespace netgen
 
 
 
-
   void Element :: ComputeIntegrationPointData () const
   {
     switch (GetType())
@@ -2255,7 +2249,6 @@ namespace netgen
 
 
 
-
   FaceDescriptor ::  FaceDescriptor()
   { 
     surfnr = domin = domout  = bcprop = 0; 
@@ -2318,13 +2311,11 @@ namespace netgen
   }
 
 
-  string FaceDescriptor :: GetBCName () const
+  const string & FaceDescriptor :: GetBCName () const
   {
-    if ( bcname )
-      return *bcname;
-    else 
-      return "default";
-  
+    static string defaultstring = "default";
+    if (bcname) return *bcname;
+    return defaultstring;
   }
 
   /*
