@@ -23,7 +23,6 @@ namespace netgen
   void MinFunctionSum :: Grad (const Vector & x, Vector & g) const
   {
     g = 0.;
-    // static Vector gi(3);
     VectorMem<3> gi;
     for(int i=0; i<functions.Size(); i++)
       {
@@ -38,7 +37,7 @@ namespace netgen
   {
     double retval = 0;
     g = 0.;
-    static Vector gi(3);
+    VectorMem<3> gi;
     for(int i=0; i<functions.Size(); i++)
       {
 	retval += functions[i]->FuncGrad(x,gi);
@@ -88,11 +87,11 @@ namespace netgen
     return *functions[i];
   }
 
-
   PointFunction1 :: PointFunction1 (Mesh::T_POINTS & apoints, 
 				    const Array<INDEX_3> & afaces,
+				    const MeshingParameters & amp,
 				    double ah)
-    : points(apoints), faces(afaces)
+    : points(apoints), faces(afaces), mp(amp)
   {
     h = ah;
   }
@@ -107,10 +106,10 @@ namespace netgen
       {
 	const INDEX_3 & el = faces[j];
 
-	double bad = CalcTetBadness (points[el.I1()], 
-				     points[el.I3()], 
-				     points[el.I2()], 
-				     pp, 0);
+	double bad = CalcTetBadness (points[PointIndex (el.I1())], 
+				     points[PointIndex (el.I3())], 
+				     points[PointIndex (el.I2())], 
+				     pp, 0, mp);
 	badness += bad;
       }
  
@@ -121,10 +120,6 @@ namespace netgen
   double PointFunction1 :: 
   FuncDeriv (const Vector & x, const Vector & dir, double & deriv) const
   {
-    /*
-    static Vector hx(3);
-    static double eps = 1e-6;
-    */
     VectorMem<3> hx;
     const double eps = 1e-6;
 
@@ -150,8 +145,8 @@ namespace netgen
 
   double PointFunction1 :: FuncGrad (const Vector & x, Vector & g) const
   {
-    static Vector hx(3);
-    static double eps = 1e-6;
+    VectorMem<3> hx;
+    double eps = 1e-6;
 
     hx = x;
     for (int i = 0; i < 3; i++)
@@ -175,8 +170,6 @@ namespace netgen
   }
 
 
-
-
   /* Cheap Functional depending of inner point inside triangular surface */
 
   // is it used ????
@@ -209,9 +202,9 @@ namespace netgen
   
     for (int i = 1; i <= nf; i++)
       {
-	const Point3d & p1 = points[faces.Get(i).I1()];
-	const Point3d & p2 = points[faces.Get(i).I2()];
-	const Point3d & p3 = points[faces.Get(i).I3()];
+	const Point3d & p1 = points[PointIndex(faces.Get(i).I1())];
+	const Point3d & p2 = points[PointIndex(faces.Get(i).I2())];
+	const Point3d & p3 = points[PointIndex(faces.Get(i).I3())];
 	Vec3d v1 (p1, p2);
 	Vec3d v2 (p1, p3);
 	Vec3d n;
@@ -247,9 +240,8 @@ namespace netgen
 
     int i;
     double badness = 0;
-    static Vector hv(4);
-    static Vector res;
-    res.SetSize (m.Height());
+    VectorMem<4> hv;
+    Vector res(m.Height());
 
     for (i = 0;i < 3; i++)
       hv(i) = vp(i);
@@ -270,8 +262,8 @@ namespace netgen
 
   double CheapPointFunction1 :: FuncGrad (const Vector & x, Vector & g) const
   {
-    static Vector hx(3);
-    static double eps = 1e-6;
+    VectorMem<3> hx;
+    double eps = 1e-6;
 
     hx = x;
     for (int i = 0; i < 3; i++)
@@ -310,12 +302,14 @@ namespace netgen
     Mesh::T_POINTS & points;
     const Mesh::T_VOLELEMENTS & elements;
     TABLE<int,PointIndex::BASE> elementsonpoint;
+    const MeshingParameters & mp;
     PointIndex actpind;
     double h;
   
   public:
     PointFunction (Mesh::T_POINTS & apoints, 
-		   const Mesh::T_VOLELEMENTS & aelements);
+		   const Mesh::T_VOLELEMENTS & aelements,
+		   const MeshingParameters & amp);
   
     virtual void SetPointIndex (PointIndex aactpind);
     void SetLocalH (double ah) { h = ah; }
@@ -329,8 +323,9 @@ namespace netgen
 
 
   PointFunction :: PointFunction (Mesh::T_POINTS & apoints, 
-				  const Mesh::T_VOLELEMENTS & aelements)
-    : points(apoints), elements(aelements), elementsonpoint(apoints.Size())
+				  const Mesh::T_VOLELEMENTS & aelements,
+				  const MeshingParameters & amp)
+    : points(apoints), elements(aelements), elementsonpoint(apoints.Size()), mp(amp)
   {
     for (int i = 0; i < elements.Size(); i++)
       if (elements[i].NP() == 4)
@@ -357,7 +352,7 @@ namespace netgen
       {
         const Element & el = elements[elementsonpoint[actpind][j]];
 	badness += CalcTetBadness (points[el[0]], points[el[1]], 
-				   points[el[2]], points[el[3]], -1);
+				   points[el[2]], points[el[3]], -1, mp);
       }
   
     points[actpind] = Point<3> (hp); 
@@ -381,7 +376,7 @@ namespace netgen
 	    {
 	      f += CalcTetBadnessGrad (points[el[0]], points[el[1]], 
                                        points[el[2]], points[el[3]], 
-                                       -1, k+1, vgradi);
+                                       -1, k+1, vgradi, mp);
 
               vgrad += vgradi;
 	    }
@@ -413,7 +408,7 @@ namespace netgen
 	      f += CalcTetBadnessGrad (points[el.PNum(1)], 
 				       points[el.PNum(2)], 
 				       points[el.PNum(3)], 
-				       points[el.PNum(4)], -1, k, vgradi);
+				       points[el.PNum(4)], -1, k, vgradi, mp);
 
 	      vgrad += vgradi;
 	    }
@@ -482,7 +477,8 @@ namespace netgen
     DenseMatrix m;
   public:
     CheapPointFunction (Mesh::T_POINTS & apoints, 
-			const Mesh::T_VOLELEMENTS & aelements);
+			const Mesh::T_VOLELEMENTS & aelements,
+			const MeshingParameters & amp);
     virtual void SetPointIndex (PointIndex aactpind);
     virtual double PointFunctionValue (const Point<3> & pp) const;
     virtual double PointFunctionValueGrad (const Point<3> & pp, Vec<3> & grad) const;
@@ -490,8 +486,9 @@ namespace netgen
 
 
   CheapPointFunction :: CheapPointFunction (Mesh::T_POINTS & apoints, 
-					    const Mesh::T_VOLELEMENTS & aelements)
-    : PointFunction (apoints, aelements)
+					    const Mesh::T_VOLELEMENTS & aelements,
+					    const MeshingParameters & amp)
+    : PointFunction (apoints, aelements, amp)
   {
     ;
   }
@@ -503,7 +500,7 @@ namespace netgen
 
     int ne = elementsonpoint[actpind].Size();
     int i, j;
-    int pi1, pi2, pi3;
+    PointIndex pi1, pi2, pi3;
 
     m.SetSize (ne, 4);
 
@@ -546,8 +543,8 @@ namespace netgen
 
   double CheapPointFunction :: PointFunctionValue (const Point<3> & pp) const
   {
-    static Vector p4(4);
-    static Vector di;
+    VectorMem<4> p4;
+    Vector di;
     int n = m.Height();
 
     p4(0) = pp(0);
@@ -574,8 +571,8 @@ namespace netgen
 
   double CheapPointFunction :: PointFunctionValueGrad (const Point<3> & pp, Vec<3> & grad) const
   {
-    static Vector p4(4);
-    static Vector di;
+    VectorMem<4> p4;
+    Vector di;
 
     int n = m.Height();
 
@@ -686,7 +683,7 @@ namespace netgen
   {
     int n = x.Size();
 
-    static Vector hx;
+    Vector hx;
     hx.SetSize(n);
 
     double eps = 1e-8;
@@ -800,7 +797,7 @@ namespace netgen
   {
     Vec3d n, vgrad;
     Point3d pp1;
-    static Vector freegrad(3);
+    VectorMem<3> freegrad;
 
     CalcNewPoint (x, pp1);
 
@@ -893,7 +890,7 @@ double Opti3EdgeMinFunction :: FuncGrad (const Vector & x, Vector & grad) const
   Vec3d n1, n2, v1, vgrad;
   Point3d pp1;
   double badness;
-  static Vector freegrad(3);
+  VectorMem<3> freegrad;
 
   CalcNewPoint (x, pp1);
 
@@ -922,22 +919,21 @@ double Opti3EdgeMinFunction :: FuncGrad (const Vector & x, Vector & grad) const
 
 
 
-extern double teterrpow;
 double CalcTotalBad (const Mesh::T_POINTS & points, 
-		     const Mesh::T_VOLELEMENTS & elements)
+		     const Mesh::T_VOLELEMENTS & elements,
+		     const MeshingParameters & mp)
 {
-  int i;
   double sum = 0;
   double elbad;
   
   tets_in_qualclass.SetSize(20);
-  for (i = 1; i <= 20; i++)
-    tets_in_qualclass.Elem(i) = 0;
+  tets_in_qualclass = 0;
 
+  double teterrpow = mp.opterrpow;
 
-  for (i = 1; i <= elements.Size(); i++)
+  for (int i = 1; i <= elements.Size(); i++)
     {
-      elbad = pow (max2(CalcBad (points, elements.Get(i), 0),1e-10),
+      elbad = pow (max2(CalcBad (points, elements.Get(i), 0, mp),1e-10),
 		   1/teterrpow);
 
       int qualclass = int (20 / elbad + 1);
@@ -1355,7 +1351,7 @@ void Mesh :: ImproveMesh (const CSGeometry & geometry, OPTIMIZEGOAL goal)
 
 
   
-void Mesh :: ImproveMesh (OPTIMIZEGOAL goal)
+void Mesh :: ImproveMesh (const MeshingParameters & mp, OPTIMIZEGOAL goal)
 {
   int typ = 1;
   
@@ -1380,14 +1376,14 @@ void Mesh :: ImproveMesh (OPTIMIZEGOAL goal)
 	  if (el.GetType() != TET)
 	    continue;
 	  
-	  double hbad = CalcBad (points, el, 0);
+	  double hbad = CalcBad (points, el, 0, mp);
 	  for (int j = 0; j < 4; j++)
 	    perrs[el[j]] += hbad;
 	  
 	  bad1 += hbad;
 	}
       
-      for (PointIndex i = PointIndex::BASE; i < np+PointIndex::BASE; i++)
+      for (int i = perrs.Begin(); i < perrs.End(); i++)
 	if (perrs[i] > badmax) 
 	  badmax = perrs[i];
       badmax = 0;
@@ -1395,7 +1391,7 @@ void Mesh :: ImproveMesh (OPTIMIZEGOAL goal)
 
   if (goal == OPT_QUALITY)
     {
-      bad1 = CalcTotalBad (points, volelements);
+      bad1 = CalcTotalBad (points, volelements, mp);
       (*testout) << "Total badness = " << bad1 << endl;
       PrintMessage (5, "Total badness = ", bad1);
     }
@@ -1410,9 +1406,9 @@ void Mesh :: ImproveMesh (OPTIMIZEGOAL goal)
   PointFunction * pf;
 
   if (typ == 1)
-    pf = new PointFunction(points, volelements);
+    pf = new PointFunction(points, volelements, mp);
   else
-    pf = new CheapPointFunction(points, volelements);
+    pf = new CheapPointFunction(points, volelements, mp);
 
   //  pf->SetLocalH (h);
   
@@ -1460,14 +1456,13 @@ void Mesh :: ImproveMesh (OPTIMIZEGOAL goal)
   const char * savetask = multithread.task;
   multithread.task = "Smooth Mesh";
   
-  for (PointIndex i = PointIndex::BASE; 
-       i < points.Size()+PointIndex::BASE; i++)
-    if ( (*this)[i].Type() == INNERPOINT && perrs[i] > 0.01 * badmax)
+  for (PointIndex pi = points.Begin(); pi < points.End(); pi++)
+    if ( (*this)[pi].Type() == INNERPOINT && perrs[pi] > 0.01 * badmax)
       {
 	if (multithread.terminate)
 	  throw NgException ("Meshing stopped");
 
-	multithread.percent = 100.0 * (i+1-PointIndex::BASE) / points.Size();
+	multithread.percent = 100.0 * (pi+1-PointIndex::BASE) / points.Size();
         /*
 	if (points.Size() < 1000)
 	  PrintDot ();
@@ -1475,14 +1470,14 @@ void Mesh :: ImproveMesh (OPTIMIZEGOAL goal)
 	  if ( (i+1-PointIndex::BASE) % 10 == 0)
 	    PrintDot ('+');
         */
-        if (  (i+1-PointIndex::BASE) % printmod == 0) PrintDot (printdot);
+        if (  (pi+1-PointIndex::BASE) % printmod == 0) PrintDot (printdot);
 
-	double lh = pointh[i];
+	double lh = pointh[pi];
 	pf->SetLocalH (lh);
 	par.typx = lh;
 
-	freeminf.SetPoint (points[i]);
-	pf->SetPointIndex (i);
+	freeminf.SetPoint (points[pi]);
+	pf->SetPointIndex (pi);
 
 	x = 0;
 	int pok;
@@ -1492,8 +1487,8 @@ void Mesh :: ImproveMesh (OPTIMIZEGOAL goal)
 	  {
 	    pok = pf->MovePointToInner ();
 
-	    freeminf.SetPoint (points[i]);
-	    pf->SetPointIndex (i);
+	    freeminf.SetPoint (points[pi]);
+	    pf->SetPointIndex (pi);
 	  }
 
 	if (pok)
@@ -1501,9 +1496,9 @@ void Mesh :: ImproveMesh (OPTIMIZEGOAL goal)
             //*testout << "start BFGS, pok" << endl;
 	    BFGS (x, freeminf, par);
             //*testout << "BFGS complete, pok" << endl;
-	    points[i](0) += x(0);
-	    points[i](1) += x(1);
-	    points[i](2) += x(2);
+	    points[pi](0) += x(0);
+	    points[pi](1) += x(1);
+	    points[pi](2) += x(2);
 	  }
       }
   PrintDot ('\n');
@@ -1515,7 +1510,7 @@ void Mesh :: ImproveMesh (OPTIMIZEGOAL goal)
 
   if (goal == OPT_QUALITY)
     {
-      bad1 = CalcTotalBad (points, volelements);
+      bad1 = CalcTotalBad (points, volelements, mp);
       (*testout) << "Total badness = " << bad1 << endl;
       PrintMessage (5, "Total badness = ", bad1);
     }
@@ -1525,7 +1520,8 @@ void Mesh :: ImproveMesh (OPTIMIZEGOAL goal)
 
 
 // Improve Condition number of Jacobian, any elements  
-void Mesh :: ImproveMeshJacobian (OPTIMIZEGOAL goal, const BitArray * usepoint)
+void Mesh :: ImproveMeshJacobian (const MeshingParameters & mp,
+				  OPTIMIZEGOAL goal, const BitArray * usepoint)
 {
   int i, j;
   
@@ -1584,9 +1580,9 @@ void Mesh :: ImproveMeshJacobian (OPTIMIZEGOAL goal, const BitArray * usepoint)
   const char * savetask = multithread.task;
   multithread.task = "Smooth Mesh Jacobian";
   
-  for (i = 1; i <= points.Size(); i++)
+  for (PointIndex pi = points.Begin(); i < points.End(); pi++)
     {
-      if ((*this)[PointIndex(i)].Type() != INNERPOINT)
+      if ((*this)[pi].Type() != INNERPOINT)
 	continue;
 
       if(usepoint && !usepoint->Test(i))
@@ -1616,7 +1612,7 @@ void Mesh :: ImproveMeshJacobian (OPTIMIZEGOAL goal, const BitArray * usepoint)
       double lh = pointh[i];
       par.typx = lh;
 
-      pf.SetPointIndex (i);
+      pf.SetPointIndex (pi);
 
       x = 0;
       int pok = (pf.Func (x) < 1e10); 
@@ -1645,7 +1641,8 @@ void Mesh :: ImproveMeshJacobian (OPTIMIZEGOAL goal, const BitArray * usepoint)
 
 
 // Improve Condition number of Jacobian, any elements  
-void Mesh :: ImproveMeshJacobianOnSurface (const BitArray & usepoint, 
+void Mesh :: ImproveMeshJacobianOnSurface (const MeshingParameters & mp,
+					   const BitArray & usepoint, 
 					   const Array< Vec<3>* > & nv,
 					   OPTIMIZEGOAL goal,
 					   const Array< Array<int,PointIndex::BASE>* > * idmaps)
@@ -1737,7 +1734,7 @@ void Mesh :: ImproveMeshJacobianOnSurface (const BitArray & usepoint,
   const char * savetask = multithread.task;
   multithread.task = "Smooth Mesh Jacobian";
   
-  for (i = 1; i <= points.Size(); i++)
+  for (PointIndex pi = points.Begin(); pi <= points.End(); pi++)
     if ( usepoint.Test(i) )
       {
 	//(*testout) << "improvejac, p = " << i << endl;
@@ -1764,9 +1761,9 @@ void Mesh :: ImproveMeshJacobianOnSurface (const BitArray & usepoint,
 	double lh = pointh[i];//GetH(points.Get(i));
 	par.typx = lh;
 
-	pf.SetPointIndex (i);
+	pf.SetPointIndex (pi);
 
-	int brother = -1;
+	PointIndex brother (-1);
 	if(usesum)
 	  {
 	    for(j=0; brother == -1 && j<used_idmaps->Size(); j++)
