@@ -1,11 +1,43 @@
 #include <mystdlib.h>
-
 #include "meshing.hpp"
 
-#define DEBUG
-
 namespace netgen
 {
+  template <class T>
+  void QuickSortRec (FlatArray<T> & data,
+		     int left, int right)
+  {
+    int i = left;
+    int j = right;
+    T midval = data[(left+right)/2];
+  
+    do
+      {
+	while (data[i] < midval) i++;
+	while (midval < data[j]) j--;
+      
+	if (i <= j)
+	  {
+	    Swap (data[i], data[j]);
+	    i++; j--;
+	  }
+      }
+    while (i <= j);
+    if (left < j) QuickSortRec (data, left, j);
+    if (i < right) QuickSortRec (data, i, right);
+  }
+
+  template <class T>
+  void QuickSort (FlatArray<T> & data)
+  {
+    if (data.Size() > 1)
+      QuickSortRec (data, 0, data.Size()-1);
+  }
+
+
+
+
+
 
   MeshTopology ::  MeshTopology (const Mesh & amesh)
     : mesh(amesh)
@@ -16,16 +48,6 @@ namespace netgen
     vert2surfelement = 0;
     vert2segment = 0;
     timestamp = -1;
-
-    edge2vert.SetName ("edge2vert");
-    face2vert.SetName ("face2vert");
-    edges.SetName ("el2edge");
-    faces.SetName ("el2face");
-    surfedges.SetName ("surfel2edge");
-    segedges.SetName ("segment2edge");
-    surffaces.SetName ("surfel2face");
-    surf2volelement.SetName ("surfel2el");
-    face2surfel.SetName ("face2surfel");
   }
 
   MeshTopology :: ~MeshTopology ()
@@ -41,9 +63,7 @@ namespace netgen
     NgProfiler::RegionTimer reg (timer);
 
 #ifdef PARALLEL
-    ParallelMeshTopology & paralleltop = mesh.GetParallelTopology();
-
-    bool isparallel = 0;
+    // ParallelMeshTopology & paralleltop = mesh.GetParallelTopology();
 #endif
 
   
@@ -57,7 +77,8 @@ namespace netgen
     int nfa = 0;
     int ned = edge2vert.Size();
 
-    PrintMessage (3, "Update mesh topology");
+    if (id == 0)
+      PrintMessage (3, "Update mesh topology");
 
     (*testout) << " UPDATE MESH TOPOLOGY " << endl; 
     (*testout) << "ne   = " << ne << endl;
@@ -85,26 +106,23 @@ namespace netgen
     for (ElementIndex ei = 0; ei < ne; ei++)
       {
 	const Element & el = mesh[ei];
-	int nelv = el.GetNV();
-	for (int j = 0; j < nelv; j++)
+	for (int j = 0; j < el.GetNV(); j++)
 	  cnt[el[j]]++;
       }
 
-    vert2element = new TABLE<int,PointIndex::BASE> (cnt);
+    vert2element = new TABLE<ElementIndex,PointIndex::BASE> (cnt);
     for (ElementIndex ei = 0; ei < ne; ei++)
       {
 	const Element & el = mesh[ei];
-	int nelv = el.GetNV();
-	for (int j = 0; j < nelv; j++)
-	  vert2element->AddSave (el[j], ei+1);
+	for (int j = 0; j < el.GetNV(); j++)
+	  vert2element->AddSave (el[j], ei);
       }
 
     cnt = 0;
     for (SurfaceElementIndex sei = 0; sei < nse; sei++)
       {
 	const Element2d & el = mesh[sei];
-	int nelv = el.GetNV();
-	for (int j = 0; j < nelv; j++)
+	for (int j = 0; j < el.GetNV(); j++)
 	  cnt[el[j]]++;
       }
 
@@ -112,8 +130,7 @@ namespace netgen
     for (SurfaceElementIndex sei = 0; sei < nse; sei++)
       {
 	const Element2d & el = mesh[sei];
-	int nelv = el.GetNV();
-	for (int j = 0; j < nelv; j++)
+	for (int j = 0; j < el.GetNV(); j++)
 	  vert2surfelement->AddSave (el[j], sei+1);
       }
 
@@ -133,14 +150,13 @@ namespace netgen
 	vert2segment->AddSave (seg[1], i);
       }
 
-  
-
     if (buildedges)
       {
 	static int timer1 = NgProfiler::CreateTimer ("topology::buildedges");
 	NgProfiler::RegionTimer reg1 (timer1);
-
-	PrintMessage (5, "Update edges ");
+	
+	if (id == 0)
+	  PrintMessage (5, "Update edges ");
       
 	edges.SetSize(ne);
 	surfedges.SetSize(nse); 
@@ -165,45 +181,28 @@ namespace netgen
 	cnt = 0;
 	for (int i = mesh.mlbetweennodes.Begin(); i < mesh.mlbetweennodes.End(); i++)
 	  {
-	    /*  JS, Oct 2009
-		int pa[2];
-		pa[0] = mesh.mlbetweennodes[i].I1();
-		pa[1] = mesh.mlbetweennodes[i].I2();
-		if (pa[0] > pa[1]) Swap (pa[0], pa[1]);
-		if (pa[0] > 0)
-		cnt.Elem(pa[0])++;
-	    */
-	    INDEX_2 parents = mesh.mlbetweennodes[i];
-	    parents.Sort();
+	    INDEX_2 parents = Sort (mesh.mlbetweennodes[i]);
 	    if (parents[0] >= PointIndex::BASE) cnt[parents[0]]++;
 	  }
-
 	TABLE<int,PointIndex::BASE> vert2vertcoarse (cnt);
 	for (int i = mesh.mlbetweennodes.Begin(); i < mesh.mlbetweennodes.End(); i++)
 	  {
-	    /*
-	      int pa[2];
-	      pa[0] = mesh.mlbetweennodes[i].I1();
-	      pa[1] = mesh.mlbetweennodes[i].I2();
-	      if (pa[0] > pa[1]) swap (pa[0], pa[1]);
-	      if (pa[0] > 0)
-	      vert2vertcoarse.AddSave1 (pa[0], pa[1]);
-	    */
-	    INDEX_2 parents = mesh.mlbetweennodes[i];
-	    parents.Sort();
+	    INDEX_2 parents = Sort (mesh.mlbetweennodes[i]);
 	    if (parents[0] > PointIndex::BASE) vert2vertcoarse.AddSave (parents[0], parents[1]);
 	  }
 
-
 	Array<int,PointIndex::BASE> edgenr(nv);
 	Array<int,PointIndex::BASE> edgeflag(nv);
-	edgeflag = 0;
+	Array<int> vertex2;
+
+	edgeflag = PointIndex::BASE-1;
 
 	ned = edge2vert.Size();
-	Array<INDEX_3> missing;
 
 	for (int i = PointIndex::BASE; i < nv+PointIndex::BASE; i++)
 	  {
+	    vertex2.SetSize (0);
+
 	    for (int j = 0; j < vert2edge[i].Size(); j++)
 	      {
 		int ednr = vert2edge[i][j];
@@ -211,22 +210,87 @@ namespace netgen
 		edgeflag[i2] = i;
 		edgenr[i2] = ednr;
 	      }
-	    for (int j = 0; j << vert2vertcoarse[i].Size(); j++)
+
+	    for (int j = 0; j < vert2vertcoarse[i].Size(); j++)    
 	      {
 		int v2 = vert2vertcoarse[i][j];
 		if (edgeflag[v2] < i)
 		  {
-		    ned++;
-		    edgenr[v2] = ned;
 		    edgeflag[v2] = i;
-		    missing.Append (INDEX_3(i,v2,ned));
+		    vertex2.Append (v2);
 		  }
 	      }
 
+	    FlatArray<ElementIndex> v2els = (*vert2element)[i];
+	    for (int j = 0; j < v2els.Size(); j++)
+	      {
+		const Element & el = mesh[v2els[j]];
+		int neledges = GetNEdges (el.GetType());
+		const ELEMENT_EDGE * eledges = GetEdges0 (el.GetType());
+		for (int k = 0; k < neledges; k++)
+		  {
+		    INDEX_2 edge(el[eledges[k][0]], el[eledges[k][1]]);
+		    edge.Sort();
+		    if (edge.I1() != i) continue;
+		    
+		    if (edgeflag[edge.I2()] < i)
+		      {
+			vertex2.Append (edge.I2());
+			edgeflag[edge.I2()] = i;
+		      }
+		  }
+	      }
+
+	    for (int j = 0; j < (*vert2surfelement)[i].Size(); j++)
+	      {
+		int elnr = (*vert2surfelement)[i][j];
+		const Element2d & el = mesh.SurfaceElement (elnr);
+
+		int neledges = GetNEdges (el.GetType());
+		const ELEMENT_EDGE * eledges = GetEdges0 (el.GetType());
+	  
+		for (int k = 0; k < neledges; k++)
+		  {
+		    INDEX_2 edge(el[eledges[k][0]], el[eledges[k][1]]);
+		    edge.Sort();
+		    if (edge.I1() != i) continue;
+	     
+		    if (edgeflag[edge.I2()] < i)
+		      {
+			vertex2.Append (edge.I2());
+			edgeflag[edge.I2()] = i;
+		      }
+		  }
+	      }
+
+	    for (int j = 0; j < (*vert2segment)[i].Size(); j++)
+	      {
+		int elnr = (*vert2segment)[i][j];
+		const Segment & el = mesh.LineSegment (elnr);
+
+		INDEX_2 edge(el[0], el[1]);
+		edge.Sort();
+		if (edge.I1() != i) continue;
+	     
+		if (edgeflag[edge.I2()] < i)
+		  {
+		    vertex2.Append (edge.I2());
+		    edgeflag[edge.I2()] = i;
+		  }   
+	      }
+
+	    QuickSort (vertex2);
+	    for (int j = 0; j < vertex2.Size(); j++)
+	      {
+		edgenr[vertex2[j]] = ++ned;
+		edge2vert.Append (INDEX_2 (i, vertex2[j]));
+	      }
+
+
 	    for (int j = 0; j < (*vert2element)[i].Size(); j++)
 	      {
-		int elnr = (*vert2element)[i][j];
-		const Element & el = mesh.VolumeElement (elnr);
+		ElementIndex elnr = (*vert2element)[i][j];
+		const Element & el = mesh[elnr];
 
 		int neledges = GetNEdges (el.GetType());
 		const ELEMENT_EDGE * eledges = GetEdges0 (el.GetType());
@@ -238,19 +302,11 @@ namespace netgen
 		    int edgedir = (edge.I1() > edge.I2());
 		    if (edgedir) swap (edge.I1(), edge.I2());
 	     
-		    if (edge.I1() != i)
-		      continue;
-	     
-		    if (edgeflag[edge.I2()] < i)
-		      {
-			ned++;
-			edgenr[edge.I2()] = ned;
-			edgeflag[edge.I2()] = i;
-		      }
+		    if (edge.I1() != i) continue;
 
 		    int edgenum = edgenr[edge.I2()];
 		    if (edgedir) edgenum *= -1;
-		    edges.Elem(elnr)[k] = edgenum;
+		    edges[elnr][k] = edgenum;
 		  }
 	      }
 
@@ -269,16 +325,8 @@ namespace netgen
 		    int edgedir = (edge.I1() > edge.I2());
 		    if (edgedir) swap (edge.I1(), edge.I2());
 	     
-		    if (edge.I1() != i)
-		      continue;
-	     
-		    if (edgeflag[edge.I2()] < i)
-		      {
-			ned++;
-			edgenr[edge.I2()] = ned;
-			edgeflag[edge.I2()] = i;
-		      }
-	      
+		    if (edge.I1() != i) continue;
+
 		    int edgenum = edgenr[edge.I2()];
 		    if (edgedir) edgenum *= -1;
 		    surfedges.Elem(elnr)[k] = edgenum;
@@ -295,384 +343,29 @@ namespace netgen
 		int edgedir = (edge.I1() > edge.I2());
 		if (edgedir) swap (edge.I1(), edge.I2());
 	      
-		if (edge.I1() != i)
-		  continue;
-	     
-		if (edgeflag[edge.I2()] < i)
-		  {
-		    ned++;
-		    edgenr[edge.I2()] = ned;
-		    edgeflag[edge.I2()] = i;
-		  }   
-		int edgenum = edgenr[edge.I2()];
+		if (edge.I1() != i) continue;
 
+		int edgenum = edgenr[edge.I2()];
 		if (edgedir) edgenum *= -1;
 		segedges.Elem(elnr) = edgenum;
 	      }
 	  }
-
-
-	edge2vert.SetSize (ned);
-	for (int i = 1; i <= ne; i++)
-	  {
-	    const Element & el = mesh.VolumeElement (i);
-      
-	    int neledges = GetNEdges (el.GetType());
-	    const ELEMENT_EDGE * eledges = GetEdges0 (el.GetType());
-	  
-	    for (int k = 0; k < neledges; k++)
-	      {
-		INDEX_2 edge(el[eledges[k][0]], el[eledges[k][1]]);
-	  
-		int edgedir = (edge.I1() > edge.I2());
-		if (edgedir) swap (edge.I1(), edge.I2());
-
-		int edgenum = abs (edges.Elem(i)[k]);
-
-		edge2vert.Elem(edgenum)[0] = edge.I1();
-		edge2vert.Elem(edgenum)[1] = edge.I2();
-	      }
-	  }
-	for (int i = 1; i <= nse; i++)
-	  {
-	    const Element2d & el = mesh.SurfaceElement (i);
-      
-	    int neledges = GetNEdges (el.GetType());
-	    const ELEMENT_EDGE * eledges = GetEdges0 (el.GetType());
-	  
-	    for (int k = 0; k < neledges; k++)
-	      {
-		INDEX_2 edge(el[eledges[k][0]], el[eledges[k][1]]);
-	  
-		int edgedir = (edge.I1() > edge.I2());
-		if (edgedir) swap (edge.I1(), edge.I2());
-
-		int edgenum = abs (surfedges.Elem(i)[k]);
-
-		edge2vert.Elem(edgenum)[0] = edge.I1();
-		edge2vert.Elem(edgenum)[1] = edge.I2();
-	      }
-	  }
-
-	for (int i = 1; i <= nseg; i++)
-	  {
-	    const Segment & el = mesh.LineSegment (i);
-      
-	    INDEX_2 edge(el[0], el[1]);
-	    int edgedir = (edge.I1() > edge.I2());
-	    if (edgedir) swap (edge.I1(), edge.I2());
-	  
-	    int edgenum = abs (segedges.Elem(i));
-	  
-	    edge2vert.Elem(edgenum)[0] = edge.I1();
-	    edge2vert.Elem(edgenum)[1] = edge.I2();
-	  }
-
-	for (int i = 1; i <= missing.Size(); i++)
-	  {
-	    INDEX_3 i3 = missing.Get(i);
-	    edge2vert.Elem(i3.I3())[0] = i3.I1();
-	    edge2vert.Elem(i3.I3())[1] = i3.I2();
-	  }
-	
-      
-	/*
-	  (*testout) << "edge table:" << endl;
-	  (*testout) << "edge2vert:" << endl;
-	  for (int i = 1; i <= edge2vert.Size(); i++)
-	  (*testout) << "edge " << i << ", v1,2 = " << edge2vert.Elem(i)[0] << ", " << edge2vert.Elem(i)[1] << endl;
-	  (*testout) << "surfedges:" << endl;
-	  for (int i = 1; i <= surfedges.Size(); i++)
-	  (*testout) << "el " << i << ", edges = " 
-	  << surfedges.Elem(i)[0] << ", "
-	  << surfedges.Elem(i)[1] << ", "
-	  << surfedges.Elem(i)[2] << endl;
-	*/ 
-     
-    
       }
 
 
-    //  cout << "build edges done" << endl;
 
     // generate faces
-    if (buildfaces) //  && mesh.GetDimension() == 3)
+    if (buildfaces) 
       {
 	static int timer2 = NgProfiler::CreateTimer ("topology::buildfaces");
 	NgProfiler::RegionTimer reg2 (timer2);
 
-	PrintMessage (5, "Update faces ");
+	if (id == 0)
+	  PrintMessage (5, "Update faces ");
 
 
 
 
-
-	/*
-
-
-	faces.SetSize(ne);
-	surffaces.SetSize(nse);
-      
-	// face2vert.SetSize(0);  // keep old faces
-	nfa = face2vert.Size();
-	// INDEX_3_HASHTABLE<int> vert2face(ne+nse+1);
-	INDEX_3_CLOSED_HASHTABLE<int> vert2face(8*ne+2*nse+nfa+2);
-
-	for (int i = 1; i <= face2vert.Size(); i++)
-	{
-	INDEX_3 f;
-	f.I1() = face2vert.Get(i)[0];
-	f.I2() = face2vert.Get(i)[1];
-	f.I3() = face2vert.Get(i)[2];
-	vert2face.Set (f, i);
-	}
-     
-	for (int i = 1; i <= ne; i++)
-	{
-	const Element & el = mesh.VolumeElement (i);
-	  
-	int nelfaces = GetNFaces (el.GetType());
-	const ELEMENT_FACE * elfaces = GetFaces1 (el.GetType());
-	  
-	for (int j = 0; j < 6; j++)
-	faces.Elem(i)[j] = 0;
-	for (int j = 0; j < nelfaces; j++)
-	if (elfaces[j][3] == 0)
-	      
-	{ // triangle
-		
-	int facenum;
-	int facedir;
-		
-	INDEX_3 face(el.PNum(elfaces[j][0]),
-	el.PNum(elfaces[j][1]),
-	el.PNum(elfaces[j][2]));
-		
-	facedir = 0;
-	if (face.I1() > face.I2())
-	{
-	swap (face.I1(), face.I2());
-	facedir += 1;
-	}
-	if (face.I2() > face.I3())
-	{
-	swap (face.I2(), face.I3());
-	facedir += 2;
-	}
-	if (face.I1() > face.I2())
-	{
-	swap (face.I1(), face.I2());
-	facedir += 4;
-	}
-		
-	if (vert2face.Used (face))
-	facenum = vert2face.Get(face);
-	else
-	{
-	nfa++;
-	vert2face.Set (face, nfa);
-	facenum = nfa;
-		    
-	INDEX_4 hface(face.I1(),face.I2(),face.I3(),0);
-	face2vert.Append (hface);
-	// face2vert.SetSize(face2vert.Size()+1);
-	}
-		
-	faces.Elem(i)[j] = 8*(facenum-1)+facedir+1;
-	}
-	  
-	else
-	      
-	{
-	// quad
-	int facenum;
-	int facedir;
-	INDEX_4Q face4(el.PNum(elfaces[j][0]),
-	el.PNum(elfaces[j][1]),
-	el.PNum(elfaces[j][2]),
-	el.PNum(elfaces[j][3]));
-		
-	facedir = 0;
-	if (min2 (face4.I1(), face4.I2()) > 
-	min2 (face4.I4(), face4.I3())) 
-	{  // z - flip
-	facedir += 1; 
-	swap (face4.I1(), face4.I4());
-	swap (face4.I2(), face4.I3());
-	}
-	if (min2 (face4.I1(), face4.I4()) >
-	min2 (face4.I2(), face4.I3())) 
-	{  // x - flip
-	facedir += 2; 
-	swap (face4.I1(), face4.I2());
-	swap (face4.I3(), face4.I4());
-	}
-	if (face4.I2() > face4.I4())
-	{  // diagonal flip
-	facedir += 4; 
-	swap (face4.I2(), face4.I4());
-	}
-	//		face4.Sort();
-		
-	INDEX_3 face(face4.I1(), face4.I2(), face4.I3());
-		
-	if (vert2face.Used (face))
-	{
-	facenum = vert2face.Get(face);
-	}
-	else
-	{
-	nfa++;
-	vert2face.Set (face, nfa);
-	facenum = nfa;
-
-	// face2vert.SetSize(face2vert.Size()+1);
-
-	INDEX_4 hface(face4.I1(),face4.I2(),face4.I3(),face4.I4());
-	face2vert.Append (hface);
-	}
-		
-	faces.Elem(i)[j] = 8*(facenum-1)+facedir+1;
-	}
-	}
-
-	face2surfel.SetSize(nfa+nse);
-	for (int i = 1; i <= face2surfel.Size(); i++)
-	face2surfel.Elem(i) = 0;
-
-	for (int i = 1; i <= nse; i++)
-	{
-	const Element2d & el = mesh.SurfaceElement (i);
-	  
-	const ELEMENT_FACE * elfaces = GetFaces1 (el.GetType());
-	  
-	if (elfaces[0][3] == 0)
-	    
-	{ // triangle
-	      
-	int facenum;
-	int facedir;
-	      
-	INDEX_3 face(el.PNum(elfaces[0][0]),
-	el.PNum(elfaces[0][1]),
-	el.PNum(elfaces[0][2]));
-	      
-	facedir = 0;
-	if (face.I1() > face.I2())
-	{
-	swap (face.I1(), face.I2());
-	facedir += 1;
-	}
-	if (face.I2() > face.I3())
-	{
-	swap (face.I2(), face.I3());
-	facedir += 2;
-	}
-	if (face.I1() > face.I2())
-	{
-	swap (face.I1(), face.I2());
-	facedir += 4;
-	}
-	      
-	if (vert2face.Used (face))
-	facenum = vert2face.Get(face);
-	else
-	{
-	nfa++;
-	vert2face.Set (face, nfa);
-	facenum = nfa;
-		  
-	// face2vert.SetSize(face2vert.Size()+1);
-	INDEX_4 hface(face.I1(),face.I2(),face.I3(),0);
-	face2vert.Append (hface);
-	}
-	      
-	surffaces.Elem(i) = 8*(facenum-1)+facedir+1;
-	face2surfel.Elem(facenum) = i;
-	}
-	  
-	else
-	    
-	{
-	// quad
-	int facenum;
-	int facedir;
-	      
-	INDEX_4Q face4(el.PNum(elfaces[0][0]),
-	el.PNum(elfaces[0][1]),
-	el.PNum(elfaces[0][2]),
-	el.PNum(elfaces[0][3]));
-
-	facedir = 0;
-	if (min2 (face4.I1(), face4.I2()) > 
-	min2 (face4.I4(), face4.I3())) 
-	{  // z - orientation
-	facedir += 1; 
-	swap (face4.I1(), face4.I4());
-	swap (face4.I2(), face4.I3());
-	}
-	if (min2 (face4.I1(), face4.I4()) >
-	min2 (face4.I2(), face4.I3())) 
-	{  // x - orientation
-	facedir += 2; 
-	swap (face4.I1(), face4.I2());
-	swap (face4.I3(), face4.I4());
-	}
-	if (face4.I2() > face4.I4())
-	{ 
-	facedir += 4; 
-	swap (face4.I2(), face4.I4());
-	}
-	      
-	INDEX_3 face(face4.I1(), face4.I2(), face4.I3());
-	      
-	if (vert2face.Used (face))
-	facenum = vert2face.Get(face);
-	else
-	{
-	nfa++;
-	vert2face.Set (face, nfa);
-	facenum = nfa;
-		  
-	// face2vert.SetSize(face2vert.Size()+1);
-	INDEX_4 hface(face4.I1(),face4.I2(),face4.I3(),face4.I3());
-	face2vert.Append (hface);
-	}
-	
-	surffaces.Elem(i) = 8*(facenum-1)+facedir+1;
-	face2surfel.Elem(facenum) = i;
-	}
-	}
-	
-	
-	surf2volelement.SetSize (nse);
-	for (int i = 1; i <= nse; i++)
-	{
-	surf2volelement.Elem(i)[0] = 0;
-	surf2volelement.Elem(i)[1] = 0;
-	}
-	for (int i = 1; i <= ne; i++)
-	for (int j = 0; j < 6; j++)
-	{
-	int fnum = (faces.Get(i)[j]+7) / 8;
-	if (fnum > 0 && face2surfel.Elem(fnum))
-	{
-	int sel = face2surfel.Elem(fnum);
-	surf2volelement.Elem(sel)[1] = 
-	surf2volelement.Elem(sel)[0];
-	surf2volelement.Elem(sel)[0] = i;
-	}
-	}
-
-	face2vert.SetAllocSize (face2vert.Size());
-	*/
-
-
-
-
-
-      
-
 	faces.SetSize(ne);
 	surffaces.SetSize(nse);
 	
@@ -699,6 +392,7 @@ namespace netgen
 	  }
 	
 
+	/*
 	for (int pass = 1; pass <= 2; pass++)
 	  {
 	    nfa = oldnfa;
@@ -715,6 +409,7 @@ namespace netgen
 		    vert2face.Set (face, fnr+1);
 		  }
 
+
 		// cout << "inherited faces: " << endl << vert2face << endl;
 
 
@@ -730,30 +425,18 @@ namespace netgen
 		      if (elfaces[j][3] == 0)
 	      
 			{ // triangle
-		      
-			  int facenum;
-			  int facedir;
-		      
+			  int facenum, facedir;
 			  INDEX_3 face(el.PNum(elfaces[j][0]),
 				       el.PNum(elfaces[j][1]),
 				       el.PNum(elfaces[j][2]));
 		      
 			  facedir = 0;
 			  if (face.I1() > face.I2())
-			    {
-			      swap (face.I1(), face.I2());
-			      facedir += 1;
-			    }
+			    { swap (face.I1(), face.I2()); facedir += 1; }
 			  if (face.I2() > face.I3())
-			    {
-			      swap (face.I2(), face.I3());
-			      facedir += 2;
-			    }
+			    { swap (face.I2(), face.I3()); facedir += 2; }
 			  if (face.I1() > face.I2())
-			    {
-			      swap (face.I1(), face.I2());
-			      facedir += 4;
-			    }
+			    { swap (face.I1(), face.I2()); facedir += 4; }
 		      
 			  if (face.I1() != v) continue;
 		      
@@ -768,7 +451,6 @@ namespace netgen
 			      INDEX_4 hface(face.I1(),face.I2(),face.I3(),0);
 			      if (pass == 2) face2vert.Append (hface);
 			    }
-			  // cout << "face = " << face << " elnr = " << elnr << endl;
 			  faces.Elem(elnr)[j] = 8*(facenum-1)+facedir+1;
 			}
 		
@@ -776,8 +458,7 @@ namespace netgen
 		    
 			{
 			  // quad
-			  int facenum;
-			  int facedir;
+			  int facenum, facedir;
 			  INDEX_4Q face4(el.PNum(elfaces[j][0]),
 					 el.PNum(elfaces[j][1]),
 					 el.PNum(elfaces[j][2]),
@@ -933,14 +614,288 @@ namespace netgen
 			  }
 		    
 			surffaces.Elem(elnr) = 8*(facenum-1)+facedir+1;
-			// face2surfel.Elem(facenum) = elnr;
 		      }
 		  }
 	      }
+
 	    face2vert.SetAllocSize (nfa);
+	    }
+	*/
+
+
+
+
+
+
+
+
+	for (int pass = 1; pass <= 2; pass++)
+	  {
+	    nfa = oldnfa;
+	    for (int v = PointIndex::BASE; v < nv+PointIndex::BASE; v++)
+	      {
+		int first_fa = nfa;
+
+		INDEX_3_CLOSED_HASHTABLE<int> vert2face(2*max_face_on_vertex+10); 
+		
+		for (int j = 0; j < vert2oldface[v].Size(); j++)
+		  {
+		    int fnr = vert2oldface[v][j];
+		    INDEX_3 face (face2vert[fnr].I1(),
+				  face2vert[fnr].I2(),
+				  face2vert[fnr].I3());
+		    vert2face.Set (face, fnr+1);
+		  }
+
+
+		if (pass == 2)
+		  for (int j = nfa; j < face2vert.Size(); j++)
+		    {
+		      if (face2vert[j][0] == v)
+			{
+			  INDEX_3 face (face2vert[j].I1(),
+					face2vert[j].I2(),
+					face2vert[j].I3());
+			  vert2face.Set (face, j+1);
+			  nfa++;
+			}
+		      else
+			break;
+		    }
+		
+
+		// cout << "inherited faces: " << endl << vert2face << endl;
+
+
+		for (int j = 0; j < (*vert2element)[v].Size(); j++)
+		  {
+		    ElementIndex elnr = (*vert2element)[v][j];
+		    const Element & el = mesh[elnr];
+	  
+		    int nelfaces = GetNFaces (el.GetType());
+		    const ELEMENT_FACE * elfaces = GetFaces1 (el.GetType());
+	  
+		    for (int j = 0; j < nelfaces; j++)
+		      if (elfaces[j][3] == 0)
+	      
+			{ // triangle
+			  int facenum, facedir;
+			  INDEX_3 face(el.PNum(elfaces[j][0]),
+				       el.PNum(elfaces[j][1]),
+				       el.PNum(elfaces[j][2]));
+		      
+			  facedir = 0;
+			  if (face.I1() > face.I2())
+			    { swap (face.I1(), face.I2()); facedir += 1; }
+			  if (face.I2() > face.I3())
+			    { swap (face.I2(), face.I3()); facedir += 2; }
+			  if (face.I1() > face.I2())
+			    { swap (face.I1(), face.I2()); facedir += 4; }
+		      
+			  if (face.I1() != v) continue;
+		      
+			  if (vert2face.Used (face))
+			    facenum = vert2face.Get(face);
+			  else
+			    {
+			      nfa++;
+			      vert2face.Set (face, nfa);
+			      facenum = nfa;
+			  
+			      INDEX_4 hface(face.I1(),face.I2(),face.I3(),0);
+			      face2vert.Append (hface);
+			    }
+			  faces[elnr][j] = 8*(facenum-1)+facedir+1;
+			}
+		
+		      else
+		    
+			{
+			  // quad
+			  int facenum, facedir;
+			  INDEX_4Q face4(el.PNum(elfaces[j][0]),
+					 el.PNum(elfaces[j][1]),
+					 el.PNum(elfaces[j][2]),
+					 el.PNum(elfaces[j][3]));
+		      
+			  facedir = 0;
+			  if (min2 (face4.I1(), face4.I2()) > 
+			      min2 (face4.I4(), face4.I3())) 
+			    {  // z - flip
+			      facedir += 1; 
+			      swap (face4.I1(), face4.I4());
+			      swap (face4.I2(), face4.I3());
+			    }
+			  if (min2 (face4.I1(), face4.I4()) >
+			      min2 (face4.I2(), face4.I3())) 
+			    {  // x - flip
+			      facedir += 2; 
+			      swap (face4.I1(), face4.I2());
+			      swap (face4.I3(), face4.I4());
+			    }
+			  if (face4.I2() > face4.I4())
+			    {  // diagonal flip
+			      facedir += 4; 
+			      swap (face4.I2(), face4.I4());
+			    }
+
+		
+			  INDEX_3 face(face4.I1(), face4.I2(), face4.I3());
+
+			  if (face.I1() != v) continue;
+		      
+			  if (vert2face.Used (face))
+			    {
+			      facenum = vert2face.Get(face);
+			    }
+			  else
+			    {
+			      nfa++;
+			      vert2face.Set (face, nfa);
+			      facenum = nfa;
+			  
+			      INDEX_4 hface(face4.I1(),face4.I2(),face4.I3(),face4.I4());
+			      face2vert.Append (hface);
+			    }
+		      
+			  faces[elnr][j] = 8*(facenum-1)+facedir+1;
+			}
+		  }
+
+		for (int j = 0; j < (*vert2surfelement)[v].Size(); j++)
+		  {
+		    int elnr = (*vert2surfelement)[v][j];
+		    // cout << "surfelnr = " << elnr << endl;
+		    const Element2d & el = mesh.SurfaceElement (elnr);
+		
+		    const ELEMENT_FACE * elfaces = GetFaces1 (el.GetType());
+		
+		    if (elfaces[0][3] == 0)
+	    
+		      { // triangle
+	      
+			int facenum;
+			int facedir;
+		    
+			INDEX_3 face(el.PNum(elfaces[0][0]),
+				     el.PNum(elfaces[0][1]),
+				     el.PNum(elfaces[0][2]));
+		    
+			// cout << "face = " << face << endl;
+
+			facedir = 0;
+			if (face.I1() > face.I2())
+			  {
+			    swap (face.I1(), face.I2());
+			    facedir += 1;
+			  }
+			if (face.I2() > face.I3())
+			  {
+			    swap (face.I2(), face.I3());
+			    facedir += 2;
+			  }
+			if (face.I1() > face.I2())
+			  {
+			    swap (face.I1(), face.I2());
+			    facedir += 4;
+			  }
+		    
+			if (face.I1() != v) continue;
+		    
+			if (vert2face.Used (face))
+			  facenum = vert2face.Get(face);
+			else
+			  {
+			    nfa++;
+			    vert2face.Set (face, nfa);
+			    facenum = nfa;
+			
+			    INDEX_4 hface(face.I1(),face.I2(),face.I3(),0);
+			    face2vert.Append (hface);
+			  }
+
+			surffaces.Elem(elnr) = 8*(facenum-1)+facedir+1;
+		      }
+	  
+		    else
+	    
+		      {
+			// quad
+			int facenum;
+			int facedir;
+		    
+			INDEX_4Q face4(el.PNum(elfaces[0][0]),
+				       el.PNum(elfaces[0][1]),
+				       el.PNum(elfaces[0][2]),
+				       el.PNum(elfaces[0][3]));
+		    
+			facedir = 0;
+			if (min2 (face4.I1(), face4.I2()) > 
+			    min2 (face4.I4(), face4.I3())) 
+			  {  // z - orientation
+			    facedir += 1; 
+			    swap (face4.I1(), face4.I4());
+			    swap (face4.I2(), face4.I3());
+			  }
+			if (min2 (face4.I1(), face4.I4()) >
+			    min2 (face4.I2(), face4.I3())) 
+			  {  // x - orientation
+			    facedir += 2; 
+			    swap (face4.I1(), face4.I2());
+			    swap (face4.I3(), face4.I4());
+			  }
+			if (face4.I2() > face4.I4())
+			  { 
+			    facedir += 4; 
+			    swap (face4.I2(), face4.I4());
+			  }
+		    
+			INDEX_3 face(face4.I1(), face4.I2(), face4.I3());
+			if (face.I1() != v) continue;
+		    
+			if (vert2face.Used (face))
+			  facenum = vert2face.Get(face);
+			else
+			  {
+			    nfa++;
+			    vert2face.Set (face, nfa);
+			    facenum = nfa;
+			
+			    INDEX_4 hface(face4.I1(),face4.I2(),face4.I3(),face4.I3());
+			    face2vert.Append (hface);
+			  }
+		    
+			surffaces.Elem(elnr) = 8*(facenum-1)+facedir+1;
+		      }
+		  }
+
+		// sort faces
+		// *testout << "faces = " << face2vert << endl;
+		if (pass == 1)
+		  {
+		    // *testout << "sort from " << first_fa << " to " << nfa << endl;
+		    for (int i = first_fa; i < nfa; i++)
+		      for (int j = first_fa+1; j < nfa; j++)
+			if (face2vert[j] < face2vert[j-1])
+			  Swap (face2vert[j-1], face2vert[j]);
+		  }
+		// *testout << "faces, sorted = " << face2vert << endl;
+	      }
+
+
+	    face2vert.SetAllocSize (nfa);
+
 	  }
 
 
+	// *testout << "face2vert = " << endl << face2vert << endl;
+
+
+
+
+
+
+
 	
 	face2surfel.SetSize (nfa);
 	face2surfel = 0;
@@ -979,39 +934,12 @@ namespace netgen
 
 	face2vert.SetAllocSize (face2vert.Size());
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 	// face table complete
 
 
-
-	/*
-	 *testout << "face2vert: ";
-	 face2vert.PrintMemInfo(cout);
-	 *testout  << "faces: ";
-	 faces.PrintMemInfo(cout);
-	 *testout << "hashtable: ";
-	 vert2face.PrintMemInfo(cout);
-	*/
-
 #ifdef PARALLEL
-	(*testout) << " RESET Paralleltop" << endl;
-
-	paralleltop.Reset ();
+	// (*testout) << " RESET Paralleltop" << endl;
+	// paralleltop.Reset ();
 #endif
 
 	Array<short int> face_els(nfa), face_surfels(nfa);
@@ -1044,48 +972,12 @@ namespace netgen
 #ifdef PARALLEL
 		    if ( ntasks > 1 )
 		      {
-			if ( !paralleltop.DoCoarseUpdate() ) continue;
-			// "illegal" faces are exchange faces
-			/*
-			  (*testout) << "exchange face : " << i << endl;
-			  (*testout) << "points = " << face2vert[i] << endl;
-			*/
-			// 		  (*testout) << "global points = ";
-			// 		  for ( int j = 0; j < 3; j++ )
-			// // 		    (*testout) << face2vert[i].I(j+1) << " -> " 
-			// // 			       << paralleltop.GetLoc2Glob_Vert( face2vert[i].I(j+1) ) << ",  ";
-			// 		  (*testout) << endl;
-			// 		  if ( !paralleltop.IsExchangeFace (i+1) )
-			// 		    paralleltop.SetRefinementFace (i+1);
-
-			paralleltop.SetExchangeFace (i+1);
-		  
-			for (int j = 0; j < 4; j++)		    
-			  {
-			    if ( face2vert[i].I(j+1) > 0 )
-			      paralleltop.SetExchangeVert(face2vert[i].I(j+1));
-			  }
-		  
-			Array<int> faceedges;
-			GetFaceEdges (i+1, faceedges);
-			for ( int j = 0; j < faceedges.Size(); j++)
-			  {
-			    paralleltop.SetExchangeEdge ( faceedges[j] );
-			    int v1, v2;
-			    GetEdgeVertices(faceedges[j], v1, v2 );
-			  }
-		  
-			/*
-			  (*testout) << "pos = ";
-			  for (int j = 0; j < 4; j++)
-			  if (face2vert[i].I(j+1) >= 1)
-			  (*testout) << mesh[(PointIndex)face2vert[i].I(j+1)] << " ";
-			  (*testout) << endl;
-			*/
+			continue;
+			// if ( !paralleltop.DoCoarseUpdate() ) continue;
 		      }
 		    else
-		      {
 #endif
+		      {
 			(*testout) << "illegal face : " << i << endl;
 			(*testout) << "points = " << face2vert[i] << endl;
 			(*testout) << "pos = ";
@@ -1094,71 +986,42 @@ namespace netgen
 			    (*testout) << mesh[(PointIndex)face2vert[i].I(j+1)] << " ";
 			(*testout) << endl;
 
-			FlatArray<int> vertels = GetVertexElements (face2vert[i].I(1));
+			FlatArray<ElementIndex> vertels = GetVertexElements (face2vert[i].I(1));
 			for (int k = 0; k < vertels.Size(); k++)
 			  {
 			    int elfaces[10], orient[10];
-			    int nf = GetElementFaces (vertels[k], elfaces, orient);
+			    int nf = GetElementFaces (vertels[k]+1, elfaces, orient);
 			    for (int l = 0; l < nf; l++)
 			      if (elfaces[l] == i)
 				{
-				  (*testout) << "is face of element " << vertels[k] << endl;
+				  // (*testout) << "is face of element " << vertels[k] << endl;
 			    
 				  if (mesh.coarsemesh && mesh.hpelements->Size() == mesh.GetNE() )
 				    {
 				      const HPRefElement & hpref_el =
-					(*mesh.hpelements) [ mesh.VolumeElement (vertels[k]).hp_elnr];
+					(*mesh.hpelements) [ mesh[vertels[k]].hp_elnr];
 				      (*testout) << "coarse eleme = " << hpref_el.coarse_elnr << endl;
 				    }
 
 				}
 			  }
-#ifdef PARALLEL
 		      }
-#endif
 		  }
 	      }
 
-#ifndef PARALLEL
-	    if (cnt_err)
-	      cout << cnt_err << " elements are not matching !!!" << endl;
-#else
 	    if (cnt_err && ntasks == 1)
 	      cout << cnt_err << " elements are not matching !!!" << endl;
-	    else if (cnt_err && ntasks > 1)
-	      {
-		cout << "p" << id << ":  " << cnt_err << " elements are not local" << endl;
-		isparallel = 1;
-	      }
-	    else if ( ntasks > 1 )
-	      cout << "p" << id << ":  " << "Partition " << id << " is totally local" << endl;
-#endif
-
 	  }
+      }
+    
 
 #ifdef PARALLEL
-     
-	if ( isparallel )
-	  {
-	    paralleltop.Update();
-	    if ( paralleltop.DoCoarseUpdate() )
-	      {
-		paralleltop.UpdateCoarseGrid();
-	      }
-	    else
-	      {
-		//  paralleltop.UpdateRefinement();
-	      }
-	    // paralleltop.Print();
-	  }
-
- 
-#endif
-
-
-
-
+    if (id != 0)  
+      {
+	// if ( paralleltop.DoCoarseUpdate() )
+	// paralleltop.UpdateCoarseGrid();
       }
+#endif
  
  
   
@@ -1410,21 +1273,18 @@ namespace netgen
 
   void MeshTopology :: GetSurfaceElementEdges (int elnr, Array<int> & eledges) const
   {
-    int i;
-    if (mesh.GetDimension()==3 || 1)
-      {
-	int ned = GetNEdges (mesh.SurfaceElement(elnr).GetType());
-	eledges.SetSize (ned);
-	for (i = 1; i <= ned; i++)
-	  eledges.Elem(i) = abs (surfedges.Get(elnr)[i-1]);
-      }
-    else
-      {
-	cout << "surfeledge(" << elnr << ") = " << flush;
-	eledges.SetSize(1); 
-	eledges.Elem(1) = abs (segedges.Get(elnr));
-	cout << eledges.Elem(1) << endl;
-      }
+    int ned = GetNEdges (mesh.SurfaceElement(elnr).GetType());
+    eledges.SetSize (ned);
+    for (int i = 0; i < ned; i++)
+      eledges[i] = abs (surfedges.Get(elnr)[i]);
+  }
+
+  void MeshTopology :: GetEdges (SurfaceElementIndex elnr, Array<int> & eledges) const
+  {
+    int ned = GetNEdges (mesh[elnr].GetType());
+    eledges.SetSize (ned);
+    for (int i = 0; i < ned; i++)
+      eledges[i] = abs (surfedges[elnr][i])-1;
   }
 
   int MeshTopology :: GetSurfaceElementFace (int elnr) const
@@ -1432,13 +1292,19 @@ namespace netgen
     return (surffaces.Get(elnr)-1) / 8 + 1;  
   }
 
+  int MeshTopology :: GetFace (SurfaceElementIndex elnr) const
+  {
+    return (surffaces[elnr]-1) / 8;  
+  }
+
+
   void MeshTopology :: 
   GetSurfaceElementEdgeOrientations (int elnr, Array<int> & eorient) const
   {
     int ned = GetNEdges (mesh.SurfaceElement(elnr).GetType());
     eorient.SetSize (ned);
-    for (int i = 1; i <= ned; i++)
-      eorient.Elem(i) = (surfedges.Get(elnr)[i-1] > 0) ? 1 : -1;
+    for (int i = 0; i < ned; i++)
+      eorient[i] = (surfedges.Get(elnr)[i] > 0) ? 1 : -1;
   }
 
   int MeshTopology :: GetSurfaceElementFaceOrientation (int elnr) const
@@ -1483,10 +1349,9 @@ namespace netgen
   void MeshTopology :: GetFaceVertices (int fnr, Array<int> & vertices) const
   {
     vertices.SetSize(4);
-    int i;
-    for (i = 1; i <= 4; i++)
-      vertices.Elem(i) = face2vert.Get(fnr)[i-1];
-    if (vertices.Elem(4) == 0)
+    for (int i = 0; i < 4; i++)
+      vertices[i] = face2vert.Get(fnr)[i];
+    if (vertices[3] == 0)
       vertices.SetSize(3);
   }
 
@@ -1503,6 +1368,12 @@ namespace netgen
     v2 = edge2vert.Get(ednr)[1];
   }
 
+  void MeshTopology :: GetEdgeVertices (int ednr, PointIndex & v1, PointIndex & v2) const
+  {
+    v1 = edge2vert.Get(ednr)[0];
+    v2 = edge2vert.Get(ednr)[1];
+  }
+
 
   void MeshTopology :: GetFaceEdges (int fnr, Array<int> & fedges, bool withorientation) const
   {
@@ -1527,12 +1398,12 @@ namespace netgen
   
 
     //  GetVertexElements (pi[0], els);
-    FlatArray<int> els= GetVertexElements (pi[0]);
+    FlatArray<ElementIndex> els = GetVertexElements (pi[0]);
 
     // find one element having all vertices of the face
     for (int i = 0; i < els.Size(); i++)
       {
-	const Element & el = mesh.VolumeElement(els[i]);
+	const Element & el = mesh[els[i]];
 	int nref_faces = GetNFaces (el.GetType());
 	const ELEMENT_FACE * ref_faces = GetFaces1 (el.GetType());
 	int nfa_ref_edges = GetNEdges (GetFaceType(fnr));
@@ -1558,7 +1429,7 @@ namespace netgen
 	  {
 	    const ELEMENT_EDGE * fa_ref_edges = GetEdges1 (GetFaceType(fnr)); 
 	    fedges.SetSize(nfa_ref_edges);
-	    GetElementEdges (els[i], eledges);
+	    GetElementEdges (els[i]+1, eledges);
 	  
 	    for (int j = 0; j < eledges.Size(); j++)
 	      {
@@ -1602,6 +1473,13 @@ namespace netgen
 	    return;
 	  }
       }   
+
+    int surfel = GetFace2SurfaceElement(fnr);
+    if (surfel != 0)
+      {
+	GetSurfaceElementEdges (surfel, fedges);
+	return;
+      }
   }
 
 
@@ -1611,24 +1489,23 @@ namespace netgen
   }
 
 
-  void MeshTopology :: GetVertexElements (int vnr, Array<int> & elements) const
+  void MeshTopology :: GetVertexElements (int vnr, Array<ElementIndex> & elements) const
   {
     if (vert2element)
       {
-	int i; 
 	int ne = vert2element->EntrySize(vnr);
 	elements.SetSize(ne);
-	for (i = 1; i <= ne; i++)
+	for (int i = 1; i <= ne; i++)
 	  elements.Elem(i) = vert2element->Get(vnr, i);
       }
   }
 
 
-  FlatArray<int> MeshTopology :: GetVertexElements (int vnr) const
+  FlatArray<ElementIndex> MeshTopology :: GetVertexElements (int vnr) const
   {
     if (vert2element)
       return (*vert2element)[vnr];
-    return FlatArray<int> (0,0);
+    return FlatArray<ElementIndex> (0,0);
   }
 
   FlatArray<int> MeshTopology :: GetVertexSurfaceElements (int vnr) const
@@ -1655,13 +1532,14 @@ namespace netgen
 
   int MeshTopology :: GetVerticesEdge ( int v1, int v2 ) const
   {
-    Array<int> elements_v1, elementedges;
+    Array<ElementIndex> elements_v1;
+    Array<int> elementedges;
     GetVertexElements ( v1, elements_v1);
     int edv1, edv2;
 
     for ( int i = 0; i < elements_v1.Size(); i++ )
       {
-	GetElementEdges( elements_v1[i], elementedges );
+	GetElementEdges( elements_v1[i]+1, elementedges );
 	for ( int ed = 0; ed < elementedges.Size(); ed ++)
 	  {
 	    GetEdgeVertices( elementedges[ed], edv1, edv2 );
@@ -1675,18 +1553,37 @@ namespace netgen
 
 
 
-  void MeshTopology :: GetSegmentVolumeElements ( int segnr, Array<int> & volels ) const
+  void MeshTopology :: 
+  GetSegmentVolumeElements ( int segnr, Array<int> & volels ) const
   {
     int v1, v2;
     GetEdgeVertices ( GetSegmentEdge (segnr), v1, v2 );
-    Array<int> volels1, volels2;
+    Array<ElementIndex> volels1, volels2;
     GetVertexElements ( v1, volels1 );
     GetVertexElements ( v2, volels2 );
     volels.SetSize(0);
 
     for ( int eli1=1; eli1 <= volels1.Size(); eli1++)
       if ( volels2.Contains( volels1.Elem(eli1) ) )
-	volels.Append ( volels1.Elem(eli1) );
-
+	volels.Append ( volels1.Elem(eli1)+1 );
   }
+
+  void MeshTopology :: 
+  GetSegmentSurfaceElements (int segnr, Array<int> & els) const
+  {
+    int v1, v2;
+    GetEdgeVertices ( GetSegmentEdge (segnr), v1, v2 );
+    Array<int> els1, els2;
+    GetVertexSurfaceElements ( v1, els1 );
+    GetVertexSurfaceElements ( v2, els2 );
+    els.SetSize(0);
+
+    for ( int eli1=1; eli1 <= els1.Size(); eli1++)
+      if ( els2.Contains( els1.Elem(eli1) ) )
+	els.Append ( els1.Elem(eli1) );
+  }
+
+
+
+
 }
