@@ -11,16 +11,14 @@ namespace netgen
 			    INDEX_2_HASHTABLE<int> & singedges,
 			    ZRefinementOptions & opt)
   {
-    int i, j;
-
     // edges selected in csg input file
-    for (i = 1; i <= geom.singedges.Size(); i++)
+    for (int i = 1; i <= geom.singedges.Size(); i++)
       {
 	//if(geom.singedges.Get(i)->maxhinit > 0)
 	//  continue; //!!!!
 
 	const SingularEdge & se = *geom.singedges.Get(i);
-	for (j = 1; j <= se.segms.Size(); j++)
+	for (int j = 1; j <= se.segms.Size(); j++)
 	  {
 	    INDEX_2 i2 = se.segms.Get(j);
 	    singedges.Set (i2, 1);
@@ -28,7 +26,7 @@ namespace netgen
       }
 
     // edges interactively selected
-    for (i = 1; i <= mesh.GetNSeg(); i++)
+    for (int i = 1; i <= mesh.GetNSeg(); i++)
       {
 	const Segment & seg = mesh.LineSegment(i);
 	if (seg.singedge_left || seg.singedge_right)
@@ -46,16 +44,14 @@ namespace netgen
   */
   void MakePrismsSingEdge (Mesh & mesh, INDEX_2_HASHTABLE<int> & singedges)
   {
-    int i, j, k;
-
     // volume elements
-    for (i = 1; i <= mesh.GetNE(); i++)
+    for (int i = 1; i <= mesh.GetNE(); i++)
       {
 	Element & el = mesh.VolumeElement(i);
 	if (el.GetType() != TET) continue;
 
-	for (j = 1; j <= 3; j++)
-	  for (k = j+1; k <= 4; k++)
+	for (int j = 1; j <= 3; j++)
+	  for (int k = j+1; k <= 4; k++)
 	    {
 	      INDEX_2 edge(el.PNum(j), el.PNum(k));
 	      edge.Sort();
@@ -80,14 +76,14 @@ namespace netgen
       }
 
     // surface elements
-    for (i = 1; i <= mesh.GetNSE(); i++)
+    for (int i = 1; i <= mesh.GetNSE(); i++)
       {
 	Element2d & el = mesh.SurfaceElement(i);
 	if (el.GetType() != TRIG) continue;
 
-	for (j = 1; j <= 3; j++)
+	for (int j = 1; j <= 3; j++)
 	  {
-	    k = (j % 3) + 1;
+	    int k = (j % 3) + 1;
 	    INDEX_2 edge(el.PNum(j), el.PNum(k));
 	    edge.Sort();
 
@@ -151,11 +147,11 @@ namespace netgen
 	  
 	    for (j = 0; j <= 1; j++)
 	      {
-		int pi1 = el.PNum( (j+0) % 4 + 1);
-		int pi2 = el.PNum( (j+1) % 4 + 1);
-		int pi3 = el.PNum( (j+2) % 4 + 1);
-		int pi4 = el.PNum( (j+3) % 4 + 1);
-		int pi5 = el.PNum(5);
+		PointIndex pi1 = el.PNum( (j+0) % 4 + 1);
+		PointIndex pi2 = el.PNum( (j+1) % 4 + 1);
+		PointIndex pi3 = el.PNum( (j+2) % 4 + 1);
+		PointIndex pi4 = el.PNum( (j+3) % 4 + 1);
+		PointIndex pi5 = el.PNum(5);
 
 		INDEX_2 edge1(pi1, pi4);
 		INDEX_2 edge2(pi2, pi3);
@@ -719,9 +715,12 @@ namespace netgen
 
 
 
-  void ZRefinement (Mesh & mesh, const CSGeometry * geom,
+  void ZRefinement (Mesh & mesh, const NetgenGeometry * hgeom,
 		    ZRefinementOptions & opt)
   {
+    const CSGeometry * geom = dynamic_cast<const CSGeometry*> (hgeom);
+    if (!geom) return;
+
     INDEX_2_HASHTABLE<int> singedges(mesh.GetNSeg());
 
     SelectSingularEdges (mesh, *geom, singedges, opt);
