@@ -1169,7 +1169,8 @@ static void findEqual (const TopTools_ListOfShape& EL1,
           for (; j<=nbj && ok; ++j) {
             if (Extrema.IsMin(j)) {
 	      hasMin = Standard_True;
-              ok = Extrema.Value(j) <= tol;
+	      ok = Extrema.Value(j) <= tol;  // V6.3
+	      // ok = Extrema.SquareDistance(j) <= tol;  // V6.5
 	    }
           }
         }
