@@ -16,10 +16,7 @@ namespace netgen
 static void STLFindEdges (STLGeometry & geom,
 			  class Mesh & mesh)
 {
-  int i, j;
-  double h;
-
-  h = mparam.maxh;
+  double h = mparam.maxh;
 
   // mark edge points:
   //int ngp = geom.GetNP();
@@ -33,7 +30,15 @@ static void STLFindEdges (STLGeometry & geom,
 
   PrintMessage(3,"Mesh Lines");
 
-  for (i = 1; i <= geom.GetNLines(); i++)
+  /*
+  cout << geom.GetNLines() << " lines" << endl;
+  double totnp = 0;
+  for (int i = 1; i <= geom.GetNLines(); i++)
+    totnp += geom.GetLine(i)->NP();
+  cout << "avg np per line " << totnp/geom.GetNLines() << endl;
+  */
+
+  for (int i = 1; i <= geom.GetNLines(); i++)
     {
       meshlines.Append(geom.GetLine(i)->Mesh(geom.GetPoints(), meshpoints, h, mesh)); 
       SetThreadPercent(100.0 * (double)i/(double)geom.GetNLines());
@@ -41,15 +46,13 @@ static void STLFindEdges (STLGeometry & geom,
 
   geom.meshpoints.SetSize(0); //testing
   geom.meshlines.SetSize(0);  //testing
-  int pim;
-  for (i = 1; i <= meshpoints.Size(); i++)
+  for (int i = 1; i <= meshpoints.Size(); i++)
     {
       geom.meshpoints.Append(meshpoints.Get(i)); //testing
-
-      pim = mesh.AddPoint(meshpoints.Get(i));
+      mesh.AddPoint(meshpoints.Get(i));
     }
   //(++++++++++++++testing
-  for (i = 1; i <= geom.GetNLines(); i++)
+  for (int i = 1; i <= geom.GetNLines(); i++)
     {
       geom.meshlines.Append(meshlines.Get(i));
     }
@@ -57,11 +60,11 @@ static void STLFindEdges (STLGeometry & geom,
 
   PrintMessage(7,"feed with edges");
 
-  for (i = 1; i <= meshlines.Size(); i++)
+  for (int i = 1; i <= meshlines.Size(); i++)
     {
       STLLine* line = meshlines.Get(i);
       (*testout) << "store line " << i << endl;
-      for (j = 1; j <= line->GetNS(); j++)
+      for (int j = 1; j <= line->GetNS(); j++)
 	{
 	  int p1, p2;
 	  
@@ -217,16 +220,6 @@ static void STLFindEdges (STLGeometry & geom,
 	  */	  
 
 	  mesh.AddSegment (seg2);
-
-
-	  /*
-	  // should be start triangle and end triangle
-	  int bothtrigs1[2] = { trig1, trig1 };
-	  meshing.AddBoundaryElement (p1, p2, sizeof (bothtrigs1), &bothtrigs1);
-	  
-	  int bothtrigs2[2] = { trig2, trig2 };
-	  meshing.AddBoundaryElement (p2, p1, sizeof (bothtrigs2), &bothtrigs2);
-	  */
 	}
     }
 
@@ -236,14 +229,12 @@ static void STLFindEdges (STLGeometry & geom,
 
 
 
-void STLSurfaceMeshing1 (STLGeometry & geom,
-			 class Mesh & mesh,
+void STLSurfaceMeshing1 (STLGeometry & geom, class Mesh & mesh,
 			 int retrynr);
 
-int STLSurfaceMeshing (STLGeometry & geom,
-		       class Mesh & mesh)
+
+int STLSurfaceMeshing (STLGeometry & geom, class Mesh & mesh)
 {
-  int i, j;
   PrintFnStart("Do Surface Meshing");
 
   geom.PrepareSurfaceMeshing();
@@ -254,15 +245,11 @@ int STLSurfaceMeshing (STLGeometry & geom,
   int nopen;
   int outercnt = 20;
 
-  //  mesh.Save ("mesh.edges");
-  
-  for (i = 1; i <= mesh.GetNSeg(); i++)
+  for (int i = 1; i <= mesh.GetNSeg(); i++)
     {
       const Segment & seg = mesh.LineSegment (i);
       if (seg.geominfo[0].trignum <= 0 || seg.geominfo[1].trignum <= 0)
-	{
-	  (*testout) << "Problem with segment " << i << ": " << seg << endl;
-	}
+	(*testout) << "Problem with segment " << i << ": " << seg << endl;
     }
 
 
@@ -270,12 +257,9 @@ int STLSurfaceMeshing (STLGeometry & geom,
     {
       outercnt--;
       if (outercnt <= 0)
-	  return MESHING3_OUTERSTEPSEXCEEDED;
-
-      if (multithread.terminate)
-	{
-	  return MESHING3_TERMINATE;
-	}
+	return MESHING3_OUTERSTEPSEXCEEDED;
+      
+      if (multithread.terminate) return MESHING3_TERMINATE;
 
       mesh.FindOpenSegments();
       nopen = mesh.GetNOpenSegments();
@@ -285,10 +269,8 @@ int STLSurfaceMeshing (STLGeometry & geom,
 	  int trialcnt = 0;
 	  while (nopen && trialcnt <= 5)
 	    {
-	      if (multithread.terminate)
-		{
-		  return MESHING3_TERMINATE;
-		}
+	      if (multithread.terminate) { return MESHING3_TERMINATE; }
+
 	      trialcnt++;
 	      STLSurfaceMeshing1 (geom, mesh, trialcnt);
 
@@ -298,14 +280,14 @@ int STLSurfaceMeshing (STLGeometry & geom,
 	      if (nopen)
 		{
 		  geom.ClearMarkedSegs();
-		  for (i = 1; i <= nopen; i++)
+		  for (int i = 1; i <= nopen; i++)
 		    {
 		      const Segment & seg = mesh.GetOpenSegment (i);
 		      geom.AddMarkedSeg(mesh.Point(seg[0]),mesh.Point(seg[1]));
 		    }
 
 		  geom.InitMarkedTrigs();
-		  for (i = 1; i <= nopen; i++)
+		  for (int i = 1; i <= nopen; i++)
 		    {
 		      const Segment & seg = mesh.GetOpenSegment (i);
 		      geom.SetMarkedTrig(seg.geominfo[0].trignum,1);
@@ -320,7 +302,7 @@ int STLSurfaceMeshing (STLGeometry & geom,
 		  mesh.CalcSurfacesOfNode();
 		  optmesh.EdgeSwapping (mesh, 0);
 		  mesh.CalcSurfacesOfNode();
-		  optmesh.ImproveMesh (mesh);
+		  optmesh.ImproveMesh (mesh, mparam);
 		}
 
 	      mesh.Compress();
@@ -361,7 +343,7 @@ int STLSurfaceMeshing (STLGeometry & geom,
 
 	      // Open edge-segments will be refined !
 	      INDEX_2_HASHTABLE<int> openseght (nopen+1);
-	      for (i = 1; i <= mesh.GetNOpenSegments(); i++)
+	      for (int i = 1; i <= mesh.GetNOpenSegments(); i++)
 		{
 		  const Segment & seg = mesh.GetOpenSegment (i);
 		  INDEX_2 i2(seg[0], seg[1]);
@@ -379,7 +361,7 @@ int STLSurfaceMeshing (STLGeometry & geom,
 	      INDEX_2_HASHTABLE<int> newpht(100);
 
 	      int nsegold = mesh.GetNSeg();
-	      for (i = 1; i <= nsegold; i++)
+	      for (int i = 1; i <= nsegold; i++)
 		{
 		  Segment seg = mesh.LineSegment(i);
 		  INDEX_2 i2(seg[0], seg[1]);
@@ -455,14 +437,11 @@ int STLSurfaceMeshing (STLGeometry & geom,
 	  PrintMessage(5,"check overlapping");
 	  // 	  mesh.FindOpenElements(); // would leed to locked points
 	  if(mesh.CheckOverlappingBoundary())
-	    {
-	      return MESHING3_BADSURFACEMESH;
-	    }
-
+	    return MESHING3_BADSURFACEMESH;
 
 	  geom.InitMarkedTrigs();
 
-	  for (i = 1; i <= mesh.GetNSE(); i++)
+	  for (int i = 1; i <= mesh.GetNSE(); i++)
 	    if (mesh.SurfaceElement(i).BadElement())
 	      {
 		int trig = mesh.SurfaceElement(i).PNum(1);
@@ -477,10 +456,10 @@ int STLSurfaceMeshing (STLGeometry & geom,
 
 	  // was commented:
 
-	  for (i = 1; i <= mesh.GetNSE(); i++)
+	  for (int i = 1; i <= mesh.GetNSE(); i++)
 	    if (mesh.SurfaceElement(i).BadElement())
 	      {
-		for (j = 1; j <= 3; j++)
+		for (int j = 1; j <= 3; j++)
 		  {
 		    refpts.Append (mesh.Point (mesh.SurfaceElement(i).PNum(j)));
 		    refh.Append (mesh.GetH (refpts.Last()) / 2);
@@ -489,7 +468,7 @@ int STLSurfaceMeshing (STLGeometry & geom,
 	      }
 	  	  
 	  // delete wrong oriented element
-	  for (i = 1; i <= mesh.GetNSE(); i++)
+	  for (int i = 1; i <= mesh.GetNSE(); i++)
 	    {
 	      const Element2d & el = mesh.SurfaceElement(i);
 	      if (!el.PNum(1))
@@ -509,7 +488,7 @@ int STLSurfaceMeshing (STLGeometry & geom,
 	    }
 	  // end comments
 
-	  for (i = 1; i <= refpts.Size(); i++)
+	  for (int i = 1; i <= refpts.Size(); i++)
 	    mesh.RestrictLocalH (refpts.Get(i), refh.Get(i));
 
 	  mesh.RemoveOneLayerSurfaceElements();
@@ -550,89 +529,153 @@ void STLSurfaceMeshing1 (STLGeometry & geom,
 			 class Mesh & mesh,
 			 int retrynr)
 {
-  int i, j;
-  double h;
-  
-  
-  h = mparam.maxh;
+  static int timer1 = NgProfiler::CreateTimer ("STL surface meshing1");
+  static int timer1a = NgProfiler::CreateTimer ("STL surface meshing1a");
+  static int timer1b = NgProfiler::CreateTimer ("STL surface meshing1b");
+  static int timer1c = NgProfiler::CreateTimer ("STL surface meshing1c");
+  static int timer1d = NgProfiler::CreateTimer ("STL surface meshing1d");
+
+  double h = mparam.maxh;
 
   mesh.FindOpenSegments();
   
   Array<int> spiralps(0);
   spiralps.SetSize(0);
-  for (i = 1; i <= geom.GetNP(); i++)
-    {
-      if (geom.GetSpiralPoint(i)) {spiralps.Append(i);}
-    }
+  for (int i = 1; i <= geom.GetNP(); i++)
+    if (geom.GetSpiralPoint(i)) 
+      spiralps.Append(i);
   
   PrintMessage(7,"NO spiralpoints = ", spiralps.Size());
   //int spfound;
-  int sppointnum;
-  int spcnt = 0;
 
+  /*
   Array<int> meshsp(mesh.GetNP());
-  for (i = 1; i <= mesh.GetNP(); i++)
+  meshsp = 0;
+  for (int i = 1; i <= mesh.GetNP(); i++)
+    for (int j = 1; j <= spiralps.Size(); j++)
+      if (Dist2(geom.GetPoint(spiralps.Get(j)), mesh.Point(i)) < 1e-20) 
+	meshsp.Elem(i) = spiralps.Get(j);
+  Array<PointIndex> imeshsp;
+  for (int i = 1; i <= meshsp.Size(); i++)
+    if (meshsp.Elem(i)) imeshsp.Append(i);
+  */
+  Array<PointIndex> imeshsp;
+  Array<int> ispiral_point;
+  for (int i = 1; i <= mesh.GetNP(); i++)
     {
-      meshsp.Elem(i) = 0;
-      for (j = 1; j <= spiralps.Size(); j++)
+      for (int j = 1; j <= spiralps.Size(); j++)
 	if (Dist2(geom.GetPoint(spiralps.Get(j)), mesh.Point(i)) < 1e-20) 
-	  meshsp.Elem(i) = spiralps.Get(j);
+	  {
+	    imeshsp.Append(i);
+	    ispiral_point.Append(spiralps.Get(j));
+	    break;
+	  }
     }
 
-
-  Array<int> opensegsperface(mesh.GetNFD());
-  for (i = 1; i <= mesh.GetNFD(); i++)
-    opensegsperface.Elem(i) = 0;
-  for (i = 1; i <= mesh.GetNOpenSegments(); i++)
-    {
-      int si = mesh.GetOpenSegment (i).si;
-      if (si >= 1 && si <= mesh.GetNFD())
-	{
-	  opensegsperface.Elem(si)++;
-	}
-      else
-	{
-	  cerr << "illegal face index" << endl;
-	}
-    }
-
-
   double starttime = GetTime ();
+  mesh.SurfaceArea().ReCalc();
+
+  int oldnp = mesh.GetNP();
+
+  Array<int,PointIndex::BASE> compress(mesh.GetNP());
+  compress = 0;
+  Array<PointIndex> icompress; 
+
+  Array<int, 1> opensegsperface(mesh.GetNFD());
+  opensegsperface = 0;
+  for (int i = 1; i <= mesh.GetNOpenSegments(); i++)
+    opensegsperface[mesh.GetOpenSegment(i).si]++;
+  
+  TABLE<int, 1> opensegments(mesh.GetNFD());
+  for (int i = 1; i <= mesh.GetNOpenSegments(); i++)
+    {
+      const Segment & seg = mesh.GetOpenSegment (i);
+      if (seg.si < 1 || seg.si > mesh.GetNFD())
+	cerr << "segment index " << seg.si << " out of range [1, " << mesh.GetNFD() << "]" << endl;
+      opensegments.Add (seg.si, i);
+    }
+  
 
   for (int fnr = 1; fnr <= mesh.GetNFD(); fnr++)
-    if (opensegsperface.Get(fnr))
-      {
-	if (multithread.terminate)
-	  return;
-	
-	PrintMessage(5,"Meshing surface ", fnr, "/", mesh.GetNFD());
-	MeshingSTLSurface meshing (geom);
-	
-	meshing.SetStartTime (starttime);
-	
-	for (i = 1; i <= mesh.GetNP(); i++)
-	  {
-	    /*
-	      spfound = 0;
-	      for (j = 1; j <= spiralps.Size(); j++)
-	      {
-	      if (Dist2(geom.GetPoint(spiralps.Get(j)),mesh.Point(i)) < 1e-20) 
-		{spfound =  1; sppointnum = spiralps.Get(j);}
+    {
+      if (fnr == 100) NgProfiler::ClearTimers();
+      if (!opensegsperface[fnr]) continue;
+      if (multithread.terminate) return;
+
+      NgProfiler::StartTimer (timer1);
+      NgProfiler::StartTimer (timer1a);
+
+
+      PrintMessage(5,"Meshing surface ", fnr, "/", mesh.GetNFD());
+      MeshingSTLSurface meshing (geom, mparam);
+      meshing.SetStartTime (starttime);
+      
+      // compress = 0;
+      icompress.SetSize(0); 
+      int cntused = 0;
+
+      for (int i = 0; i < imeshsp.Size(); i++)
+	{
+	  compress[imeshsp[i]] = ++cntused;
+	  icompress.Append(imeshsp[i]);
+	}
+
+      NgProfiler::StopTimer (timer1a);
+      NgProfiler::StartTimer (timer1b);
+
+
+
+      /*
+      for (int i = 1; i <= mesh.GetNOpenSegments(); i++)
+	{
+	  const Segment & seg = mesh.GetOpenSegment (i);
+	  if (seg.si == fnr)
+	    for (int j = 0; j < 2; j++)
+	      if (compress[seg[j]] == 0)
+		{
+		  compress[seg[j]] = ++cntused;
+		  icompress.Append(seg[j]);
 		}
-	    */
-	    sppointnum = 0;
-	    if (i <= meshsp.Size())
-	      sppointnum = meshsp.Get(i);
-	    
-	  //spfound = 0;
+	}
+      */
+      FlatArray<int> segs = opensegments[fnr];
+      for (int hi = 0; hi < segs.Size(); hi++)
+	{
+	  int i = segs[hi];
+	  const Segment & seg = mesh.GetOpenSegment (i);
+	  for (int j = 0; j < 2; j++)
+	    if (compress[seg[j]] == 0)
+	      {
+		compress[seg[j]] = ++cntused;
+		icompress.Append(seg[j]);
+	      }
+	}
+
+      NgProfiler::StopTimer (timer1b);
+      NgProfiler::StartTimer (timer1c);
+
+
+      for (int hi = 0; hi < icompress.Size(); hi++)
+	{
+	  PointIndex pi = icompress[hi];
+	  
+	  /*
+	  // int sppointnum = meshsp.Get(i);
+	  int sppointnum = 0;
+	  if (hi < ispiral_point.Size())
+	    sppointnum = ispiral_point[hi];
+
 	  if (sppointnum)
 	    {
+	  */
+	  if (hi < ispiral_point.Size())
+	    {
+	      int sppointnum = ispiral_point[hi];
+
 	      MultiPointGeomInfo mgi;
-  
-	      int ntrigs = geom.NOTrigsPerPoint(sppointnum);
-	      spcnt++;
 	      
-	      for (j = 0; j < ntrigs; j++)
+	      int ntrigs = geom.NOTrigsPerPoint(sppointnum);
+	      for (int j = 0; j < ntrigs; j++)
 		{
 		  PointGeomInfo gi;
 		  gi.trignum = geom.TrigPerPoint(sppointnum, j+1);
@@ -643,35 +686,55 @@ void STLSurfaceMeshing1 (STLGeometry & geom,
 	      // Dreiecke (werden dann intern kopiert)
 	      // Ein Segment zum ConePoint muss vorhanden sein !!!
 	      
-	      meshing.AddPoint (mesh.Point(i), i, &mgi);
-	      
+	      // meshing.AddPoint (mesh.Point(i), i, &mgi);
+	      meshing.AddPoint (mesh[pi], pi, &mgi);
 	    }
 	  else
-	    {
-	      meshing.AddPoint (mesh.Point(i), i);
-	    }
+	    meshing.AddPoint (mesh[pi], pi);
 	}
-      
-      
-      for (i = 1; i <= mesh.GetNOpenSegments(); i++)
-	{
-	  const Segment & seg = mesh.GetOpenSegment (i);
-	  if (seg.si == fnr)
-	    meshing.AddBoundaryElement (seg[0], seg[1], seg.geominfo[0], seg.geominfo[1]);
-	}
-      
-      
-      PrintMessage(3,"start meshing, trialcnt = ", retrynr);
+
+      NgProfiler::StopTimer (timer1c);
+      NgProfiler::StartTimer (timer1d);
 
       /*
-      (*testout) << "start meshing with h = " << h << endl;
+        for (int i = 1; i <= mesh.GetNOpenSegments(); i++)
+	  {
+	    const Segment & seg = mesh.GetOpenSegment (i);
+	    if (seg.si == fnr)
+	      meshing.AddBoundaryElement (compress[seg[0]], compress[seg[1]], 
+					  seg.geominfo[0], seg.geominfo[1]);
+	  }
       */
-      meshing.GenerateMesh (mesh, h, fnr);  // face index
 
+
+      // FlatArray<int> segs = opensegments[fnr];
+      for (int hi = 0; hi < segs.Size(); hi++)
+	{
+	  int i = segs[hi];
+	  const Segment & seg = mesh.GetOpenSegment (i);
+	  meshing.AddBoundaryElement (compress[seg[0]], compress[seg[1]], 
+				      seg.geominfo[0], seg.geominfo[1]);
+	}
+
+
+
+      NgProfiler::StopTimer (timer1d);
+
+      NgProfiler::StopTimer (timer1);
+      
+      PrintMessage(3,"start meshing, trialcnt = ", retrynr);
+      
+      meshing.GenerateMesh (mesh, mparam, h, fnr);  
+      
+      for (int i = 0; i < icompress.Size(); i++)
+	compress[icompress[i]] = 0;
+      
+      
       extern void Render();
       Render();
-    }    
-      
+    }     
+  
+  // NgProfiler::Print(stdout);
   
   mesh.CalcSurfacesOfNode();
 }
@@ -684,19 +747,7 @@ void STLSurfaceOptimization (STLGeometry & geom,
 {
   PrintFnStart("optimize STL Surface");
 
-
   MeshOptimizeSTLSurface optmesh(geom);
-  //
-
-  int i;
-  /*
-  for (i = 1; i <= mparam.optsteps2d; i++)
-    {
-      EdgeSwapping (mesh, 1, 1);
-      CombineImprove (mesh, 1);
-      optmesh.ImproveMesh (mesh, 0, 10, 1, 1);
-    }
-  */
 
   optmesh.SetFaceIndex (0);
   optmesh.SetImproveEdges (0);
@@ -704,7 +755,7 @@ void STLSurfaceOptimization (STLGeometry & geom,
 
   PrintMessage(5,"optimize string = ", meshparam.optimize2d, " elsizew = ", meshparam.elsizeweight);
 
-  for (i = 1; i <= meshparam.optsteps2d; i++)
+  for (int i = 1; i <= meshparam.optsteps2d; i++)
     for (size_t j = 1; j <= strlen(meshparam.optimize2d); j++)
       {
 	if (multithread.terminate)
@@ -727,7 +778,7 @@ void STLSurfaceOptimization (STLGeometry & geom,
 	    }
 	  case 'm': 
 	    {
-	      optmesh.ImproveMesh(mesh);
+	      optmesh.ImproveMesh(mesh, mparam);
 	      break;
 	    }
 	  case 'c': 
@@ -749,8 +800,9 @@ void STLSurfaceOptimization (STLGeometry & geom,
 
 
 
-MeshingSTLSurface :: MeshingSTLSurface (STLGeometry & ageom)
-  : Meshing2(ageom.GetBoundingBox()), geom(ageom)
+MeshingSTLSurface :: MeshingSTLSurface (STLGeometry & ageom,
+					const MeshingParameters & mp)
+  : Meshing2(mp, ageom.GetBoundingBox()), geom(ageom)
 {
   ;
 }
@@ -768,14 +820,13 @@ void MeshingSTLSurface :: TransformToPlain (const Point3d & locpoint, const Mult
 					    Point2d & plainpoint, double h, int & zone)
 {
   int trigs[10000];
-  int i;
 
   if (gi.GetNPGI() >= 9999) 
     {
       PrintError("In Transform to plane: increase size of trigs!!!");
     }
 
-  for (i = 1; i <= gi.GetNPGI(); i++)
+  for (int i = 1; i <= gi.GetNPGI(); i++)
     trigs[i-1] = gi.GetPGI(i).trignum;
   trigs[gi.GetNPGI()] = 0;
 
@@ -840,9 +891,7 @@ int MeshingSTLSurface ::
 ChooseChartPointGeomInfo (const MultiPointGeomInfo & mpgi, 
 			  PointGeomInfo & pgi)
 {
-  int i;
-
-  for (i = 1; i <= mpgi.GetNPGI(); i++)
+  for (int i = 1; i <= mpgi.GetNPGI(); i++)
     if (geom.TrigIsInOC (mpgi.GetPGI(i).trignum, geom.meshchart))
       {
 	pgi = mpgi.GetPGI(i);
@@ -978,10 +1027,7 @@ int  MeshOptimizeSTLSurface :: CalcPointGeomInfo(PointGeomInfo& gi, const Point<
   Point<3> hp = p3;
   gi.trignum = geom.Project (hp);
 
-  if (gi.trignum)
-    {
-      return 1;
-    }
+  if (gi.trignum) return 1;
 
   return 0;
   
@@ -990,13 +1036,6 @@ int  MeshOptimizeSTLSurface :: CalcPointGeomInfo(PointGeomInfo& gi, const Point<
 void MeshOptimizeSTLSurface :: GetNormalVector(INDEX surfind, const Point<3> & p, Vec<3> & n) const
 {
   n = geom.GetChartNormalVector();
-  
-  /*
-  geometry.GetSurface(surfind)->CalcGradient (p, n);
-  n /= n.Length();
-  if (geometry.GetSurface(surfind)->Inverse())
-    n *= -1;
-  */
 }
   
 
@@ -1024,7 +1063,7 @@ PointBetween  (const Point<3> & p1, const Point<3> & p2, double secpoint,
 	       int surfi, 
 	       const PointGeomInfo & gi1, 
 	       const PointGeomInfo & gi2,
-	       Point<3> & newp, PointGeomInfo & newgi)
+	       Point<3> & newp, PointGeomInfo & newgi) const
 {
   newp = p1+secpoint*(p2-p1);
 
@@ -1087,7 +1126,7 @@ PointBetween (const Point<3> & p1, const Point<3> & p2, double secpoint,
 	      int surfi1, int surfi2, 
 	      const EdgePointGeomInfo & gi1, 
 	      const EdgePointGeomInfo & gi2,
-	      Point<3> & newp, EdgePointGeomInfo & newgi)
+	      Point<3> & newp, EdgePointGeomInfo & newgi) const
 {
   /*
   (*testout) << "edge-between: p1 = " << p1 << ", p2 = " << p2
@@ -1113,14 +1152,14 @@ PointBetween (const Point<3> & p1, const Point<3> & p2, double secpoint,
 }
 
 
-void RefinementSTLGeometry :: ProjectToSurface (Point<3> & p, int surfi)
+void RefinementSTLGeometry :: ProjectToSurface (Point<3> & p, int surfi) const
 {
   cout << "RefinementSTLGeometry :: ProjectToSurface not implemented!" << endl;
 };
 
 
 void RefinementSTLGeometry :: ProjectToSurface (Point<3> & p, int surfi,
-						PointGeomInfo & gi)
+						PointGeomInfo & gi) const
 {
   ((STLGeometry&)geom).SelectChartOfTriangle (gi.trignum);
   gi.trignum = geom.Project (p);
