@@ -1,14 +1,7 @@
-#include <mystdlib.h>
-
-#include <myadt.hpp>
-#include <linalg.hpp>
-#include <gprim.hpp>
-
 #include <meshing.hpp>
 
 #include "stlgeom.hpp"
 
-
 namespace netgen
 {
 
@@ -39,14 +32,16 @@ void STLMeshing (STLGeometry & geom,
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 
-STLGeometry :: STLGeometry()
+  STLGeometry :: STLGeometry()
+  /*
   : edges(), edgesperpoint(),
     normals(),  externaledges(),
     atlas(), chartmark(), 
     lines(), outerchartspertrig(), vicinity(), markedtrigs(), markedsegs(),
     lineendpoints(), spiralpoints(), selectedmultiedge()
+  */
 {
-	edgedata = new STLEdgeDataList(*this);
+  edgedata = new STLEdgeDataList(*this);
   externaledges.SetSize(0);
   Clear();
   meshchart = 0; // initialize all ?? JS
@@ -60,6 +55,7 @@ STLGeometry :: STLGeometry()
   status = STL_GOOD;
   statustext = "Good Geometry";
   smoothedges = NULL;
+  area = -1;
 }
 
 STLGeometry :: ~STLGeometry()
@@ -67,16 +63,42 @@ STLGeometry :: ~STLGeometry()
   delete edgedata;
 }
 
-int STLGeometry :: GenerateMesh (Mesh*& mesh,
-				 int perfstepsstart, int perfstepsend, char* optstring)
+void STLGeometry :: Save (string filename) const
 {
-  return STLMeshingDummy (this, mesh, perfstepsstart, perfstepsend, optstring);
+  const char  * cfilename = filename.c_str();
+  if (strlen(cfilename) < 4) 
+    throw NgException ("illegal filename");
+
+  if (strlen(cfilename) > 3 &&
+      strcmp (&cfilename[strlen(cfilename)-3], "stl") == 0)
+    {
+      STLTopology::Save (cfilename);
+    }
+  else if (strlen(cfilename) > 4 &&
+	   strcmp (&cfilename[strlen(cfilename)-4], "stlb") == 0)
+    {
+      SaveBinary (cfilename,"Binary STL Geometry");
+      
+    }
+  else if (strlen(cfilename) > 4 &&
+	   strcmp (&cfilename[strlen(cfilename)-4], "stle") == 0)
+    {
+      SaveSTLE (cfilename);
+    }
+}
+
+
+
+int STLGeometry :: GenerateMesh (Mesh*& mesh, MeshingParameters & mparam,
+				 int perfstepsstart, int perfstepsend)
+{
+  return STLMeshingDummy (this, mesh, mparam, perfstepsstart, perfstepsend);
 }
 
 
 const Refinement & STLGeometry :: GetRefinement () const
 {
-  return * new RefinementSTLGeometry (*this);
+  return RefinementSTLGeometry (*this);
 }
 
 
@@ -1983,13 +2005,11 @@ void STLGeometry :: Clear()
 
 double STLGeometry :: Area()
 {
-  double ar = 0;
-  int i;
-  for (i = 1; i <= GetNT(); i++)
-    {
-      ar += GetTriangle(i).Area(points);
-    }
-  return ar;
+  if (area >= 0) return area;
+  area = 0;
+  for (int i = 1; i <= GetNT(); i++)
+    area += GetTriangle(i).Area(points);
+  return area;
 }
 
 double STLGeometry :: GetAngle(int t1, int t2)
@@ -2083,22 +2103,21 @@ void STLGeometry :: TopologyChanged()
 
 int STLGeometry :: CheckGeometryOverlapping()
 {
-  int i, j, k;
+  PrintMessageCR(3,"Check overlapping geometry ...");
 
   Box<3> geombox = GetBoundingBox();
   Point<3> pmin = geombox.PMin();
   Point<3> pmax = geombox.PMax();
 
   Box3dTree setree(pmin, pmax);
-  Array<int> inters;
 
   int oltrigs = 0;
   markedtrigs.SetSize(GetNT());
 
-  for (i = 1; i <= GetNT(); i++)
+  for (int i = 1; i <= GetNT(); i++)
     SetMarkedTrig(i, 0);
 
-  for (i = 1; i <= GetNT(); i++)
+  for (int i = 1; i <= GetNT(); i++)
     {
       const STLTriangle & tri = GetTriangle(i);
       
@@ -2112,48 +2131,57 @@ int STLGeometry :: CheckGeometryOverlapping()
       setree.Insert (tpmin, tpmax, i);
     }
 
-  for (i = 1; i <= GetNT(); i++)
-    {
-      const STLTriangle & tri = GetTriangle(i);
-      
-      Point<3> tpmin = tri.box.PMin();
-      Point<3> tpmax = tri.box.PMax();
 
-      setree.GetIntersecting (tpmin, tpmax, inters);
-
-      for (j = 1; j <= inters.Size(); j++)
-	{
-	  const STLTriangle & tri2 = GetTriangle(inters.Get(j));
-
-	  const Point<3> *trip1[3], *trip2[3];	
-	  Point<3> hptri1[3], hptri2[3];
-	  /*
-	  for (k = 1; k <= 3; k++)
-	    {
+#pragma omp parallel
+  {
+    Array<int> inters;
+    
+#pragma omp for
+    for (int i = 1; i <= GetNT(); i++)
+      {
+	const STLTriangle & tri = GetTriangle(i);
+	
+	Point<3> tpmin = tri.box.PMin();
+	Point<3> tpmax = tri.box.PMax();
+	
+	setree.GetIntersecting (tpmin, tpmax, inters);
+	
+	for (int j = 1; j <= inters.Size(); j++)
+	  {
+	    const STLTriangle & tri2 = GetTriangle(inters.Get(j));
+	    
+	    const Point<3> *trip1[3], *trip2[3];	
+	    Point<3> hptri1[3], hptri2[3];
+	    /*
+	      for (k = 1; k <= 3; k++)
+	      {
 	      trip1[k-1] = &GetPoint (tri.PNum(k));
 	      trip2[k-1] = &GetPoint (tri2.PNum(k));
-	    }
-	  */
+	      }
+	    */
+	    
+	    for (int k = 0; k < 3; k++)
+	      {
+		hptri1[k] = GetPoint (tri[k]);
+		hptri2[k] = GetPoint (tri2[k]);
+		trip1[k] = &hptri1[k];
+		trip2[k] = &hptri2[k];
+	      }
 
-	  for (k = 0; k < 3; k++)
-	    {
-	      hptri1[k] = GetPoint (tri[k]);
-	      hptri2[k] = GetPoint (tri2[k]);
-	      trip1[k] = &hptri1[k];
-	      trip2[k] = &hptri2[k];
-	    }
-
-	  if (IntersectTriangleTriangle (&trip1[0], &trip2[0]))
-	    {
-	      oltrigs++;
-	      PrintMessage(5,"Intersecting Triangles: trig ",i," with ",inters.Get(j),"!");
-	      SetMarkedTrig(i, 1);
-	      SetMarkedTrig(inters.Get(j), 1);
-	    }
-	}
-    }
-
-  PrintMessage(3,"Check Geometry Overlapping: overlapping triangles = ",oltrigs);
+	    if (IntersectTriangleTriangle (&trip1[0], &trip2[0]))
+	      {
+#pragma omp critical
+		{
+		  oltrigs++;
+		  PrintMessage(5,"Intersecting Triangles: trig ",i," with ",inters.Get(j),"!");
+		  SetMarkedTrig(i, 1);
+		  SetMarkedTrig(inters.Get(j), 1);
+		}
+	      }
+	  }
+      }
+  }
+  PrintMessage(3,"Check overlapping geometry ... ", oltrigs, " triangles overlap");
   return oltrigs;
 }
 
@@ -2387,12 +2415,11 @@ void STLGeometry :: CalcEdgeData()
   PushStatus("Calc Edge Data");
 
   int np1, np2;
-  int i;
   
   int ecnt = 0;
   edgedata->SetSize(GetNT()/2*3);
 
-  for (i = 1; i <= GetNT(); i++)
+  for (int i = 1; i <= GetNT(); i++)
     {
       SetThreadPercent((double)i/(double)GetNT()*100.);
       
@@ -2431,11 +2458,9 @@ void STLGeometry :: CalcEdgeData()
 
 void STLGeometry :: CalcEdgeDataAngles()
 {
-  PrintMessage(5,"calc edge data angles");
+  PrintMessageCR (5,"calc edge data angles ... ");
 
-  int i;
-
-  for (i = 1; i <= GetNTE(); i++)
+  for (int i = 1; i <= GetNTE(); i++)
     {
       STLTopEdge & edge = GetTopEdge (i);
       double cosang = 
@@ -2444,7 +2469,7 @@ void STLGeometry :: CalcEdgeDataAngles()
       edge.SetCosAngle (cosang);
     }
 
-  for (i = 1; i <= edgedata->Size(); i++)
+  for (int i = 1; i <= edgedata->Size(); i++)
     {
       /*
       const STLEdgeData& e = edgedata->Get(i);
@@ -2453,7 +2478,7 @@ void STLGeometry :: CalcEdgeDataAngles()
       edgedata->Elem(i).angle = fabs(ang);
       */
     }
-  
+  PrintMessage (5,"calc edge data angles ... done");
 }
 
 void STLGeometry :: FindEdgesFromAngles()
@@ -3023,11 +3048,10 @@ void STLGeometry :: BuildSmoothEdges ()
 
   PushStatusF("Build Smooth Edges");
 
-  int i, j;//, k, l;
   int nt = GetNT();
   Vec3d ng1, ng2;
 
-  for (i = 1; i <= nt; i++)
+  for (int i = 1; i <= nt; i++)
     {
       if (multithread.terminate)
 	{PopStatus();return;}
@@ -3039,7 +3063,7 @@ void STLGeometry :: BuildSmoothEdges ()
       ng1 = trig.GeomNormal(points);
       ng1 /= (ng1.Length() + 1e-24);
 
-      for (j = 1; j <= 3; j++)
+      for (int j = 1; j <= 3; j++)
 	{ 
 	  int nbt = NeighbourTrig (i, j);
 	  
@@ -3063,7 +3087,6 @@ void STLGeometry :: BuildSmoothEdges ()
 	    }
 	}
     }
-
   PopStatus();
 }
 
@@ -3082,7 +3105,7 @@ int STLGeometry :: IsSmoothEdge (int pi1, int pi2) const
 
 
 
-
+/*
 //function is not used now
 int IsInArray(int n, const Array<int>& ia)
 {
@@ -3093,6 +3116,7 @@ int IsInArray(int n, const Array<int>& ia)
     }
   return 0;
 }
+*/
 
 void STLGeometry :: AddConeAndSpiralEdges()
 {
@@ -3482,4 +3506,65 @@ void STLGeometry :: SmoothGeometry ()
       SetPoint (i, pi);
     }
 }
+
+
+
+  class STLGeometryRegister : public GeometryRegister
+  {
+  public:
+    virtual NetgenGeometry * Load (string filename) const;
+  };
+
+  NetgenGeometry *  STLGeometryRegister :: Load (string filename) const
+  {
+    const char * cfilename = filename.c_str();
+
+    if (strcmp (&cfilename[strlen(cfilename)-3], "stl") == 0)
+      {
+	PrintMessage (1, "Load STL geometry file ", cfilename);
+
+	ifstream infile(cfilename);
+
+	STLGeometry * hgeom = STLGeometry :: Load (infile);
+	hgeom -> edgesfound = 0;
+	return hgeom;
+      }
+    else if (strcmp (&cfilename[strlen(cfilename)-4], "stlb") == 0)
+      {
+	PrintMessage (1, "Load STL binary geometry file ", cfilename);
+
+	ifstream infile(cfilename);
+
+	STLGeometry * hgeom = STLGeometry :: LoadBinary (infile);
+	hgeom -> edgesfound = 0;
+	return hgeom;
+      }
+    else if (strcmp (&cfilename[strlen(cfilename)-3], "nao") == 0)
+      {
+	PrintMessage (1, "Load naomi (F. Kickinger) geometry file ", cfilename);
+
+	ifstream infile(cfilename);
+
+	STLGeometry * hgeom = STLGeometry :: LoadNaomi (infile);
+	hgeom -> edgesfound = 0;
+	return hgeom;
+      }
+
+    
+    return NULL;
+  }
+
+
+  class STLInit
+  {
+  public:
+    STLInit()
+    {
+      geometryregister.Append (new STLGeometryRegister);
+    }
+  };
+
+  STLInit stlinit;
+
+
 }
