@@ -5,7 +5,7 @@
 namespace netgen
 {
 
-
+  /*
   extern void InitDrawMesh ();
   extern void DrawMesh ();
   extern void MouseMove(int oldx, int oldy,
@@ -13,26 +13,26 @@ namespace netgen
                         char mode);
 
   extern void Render ();
-
-
+  */
 
 
   class VisualScene
   {
   protected:
-    static Point3d center;
-    static double rad;
+    static DLL_HEADER Point3d center;
+    static DLL_HEADER double rad;
 
     static float lookatmat[16];
     static float transmat[16];
     static float rotmat[16];
     static float centermat[16];
-    static float transformationmat[16];
+	
+    static DLL_HEADER float transformationmat[16];
 
     GLdouble clipplane[4];
 
     int changeval;
-    static GLdouble backcolor;
+    static DLL_HEADER GLdouble backcolor;
 
     static int selface;
     static int selelement;
@@ -44,7 +44,7 @@ namespace netgen
     static int selecttimestamp;
 
   public:
-
+	static int viewport[4];
     // static GLubyte * colortexture;
     static GLuint coltexname;
     static int ntexcols;
@@ -53,104 +53,47 @@ namespace netgen
 
 
   public:
-    VisualScene ();
-    virtual ~VisualScene();
+    DLL_HEADER VisualScene ();
+    DLL_HEADER virtual ~VisualScene();
 
-    virtual void BuildScene (int zoomall = 0);
-    virtual void DrawScene ();
+    DLL_HEADER virtual void BuildScene (int zoomall = 0);
+    DLL_HEADER virtual void DrawScene ();
   
-    void CalcTransformationMatrices();
-    void StandardRotation (const char * dir);
-    void ArbitraryRotation (const Array<double> & alpha, const Array<Vec3d> & vec);
-    void ArbitraryRotation (const double alpha, const Vec3d & vec);
+    DLL_HEADER void CalcTransformationMatrices();
+    DLL_HEADER void StandardRotation (const char * dir);
+    DLL_HEADER void ArbitraryRotation (const Array<double> & alpha, const Array<Vec3d> & vec);
+    DLL_HEADER void ArbitraryRotation (const double alpha, const Vec3d & vec);
 
-    void MouseMove(int oldx, int oldy,
+    DLL_HEADER void MouseMove(int oldx, int oldy,
                    int newx, int newy,
                    char mode);
 
-    void LookAt (const Point<3> & cam, const Point<3> & obj,
+    DLL_HEADER void LookAt (const Point<3> & cam, const Point<3> & obj,
                  const Point<3> & camup);
 
-    void SetClippingPlane ();
+    DLL_HEADER void SetClippingPlane ();
 
-    virtual void MouseDblClick (int px, int py);
+    DLL_HEADER virtual void MouseDblClick (int px, int py);
 
-    void SetLight ();
+    DLL_HEADER void SetLight ();
     static void SetBackGroundColor (double col)
     { backcolor = col; }
 
-    void CreateTexture (int ncols, int linear, int typ = GL_DECAL);
-    void DrawColorBar (double minval, double maxval, int logscale = 0, bool linear = 1);
-    void DrawCoordinateCross ();
-    void DrawNetgenLogo ();
-    void SetOpenGlColor(double val, double valmin, double valmax, int logscale = 0);
-
+    DLL_HEADER void CreateTexture (int ncols, int linear, double alpha, int typ);
+    DLL_HEADER void DrawColorBar (double minval, double maxval, int logscale = 0, bool linear = 1);
+    DLL_HEADER void DrawCoordinateCross ();
+    DLL_HEADER void DrawNetgenLogo ();
+    DLL_HEADER void SetOpenGlColor(double val, double valmin, double valmax, int logscale = 0);
+	
 
 #ifdef PARALLELGL
-    void InitParallelGL ();
-    void Broadcast ();
+    DLL_HEADER void InitParallelGL ();
+    DLL_HEADER void Broadcast ();
 #endif 
   };
 
 
-  extern void MyOpenGLText (const char * text);
-
-
-
-  class VisualSceneGeometry : public VisualScene
-  {
-    Array<int> trilists;
-    int selsurf;
-  public:
-    VisualSceneGeometry ();
-    virtual ~VisualSceneGeometry ();
-
-    virtual void SelectSurface (int aselsurf);
-    virtual void BuildScene (int zoomall = 0);
-    virtual void DrawScene ();
-  };
-
-
-
-  class VisualSceneSTLGeometry : public VisualScene
-  {
-    Array<int> trilists;
-  
-  public:
-    VisualSceneSTLGeometry ();
-    virtual ~VisualSceneSTLGeometry ();
-
-    virtual void BuildScene (int zoomall = 0);
-    virtual void DrawScene ();
-  };
-
-
-  class VisualSceneGeometry2d : public VisualScene
-  {
-  public:
-    VisualSceneGeometry2d ();
-    virtual ~VisualSceneGeometry2d ();
-
-    virtual void BuildScene (int zoomall = 0);
-    virtual void DrawScene ();
-  };
-
-
-#ifdef OCCGEOMETRY
-  class VisualSceneOCCGeometry : public VisualScene
-  {
-    Array<int> trilists;
-    Array<int> linelists;
-    int selsurf;
-  public:
-    VisualSceneOCCGeometry ();
-    virtual ~VisualSceneOCCGeometry ();
-
-    virtual void BuildScene (int zoomall = 0);
-    virtual void DrawScene ();
-    virtual void MouseDblClick (int px, int py);
-  };
-#endif
+  DLL_HEADER extern void MyOpenGLText (const char * text);
 
 
 
@@ -161,39 +104,6 @@ namespace netgen
 
 
 
-#ifdef STEP
-  class VisualSceneSTEPGeometry : public VisualScene
-  {
-    Array<int> gllists;
-  
-  public:
-    VisualSceneSTEPGeometry ();
-    virtual ~VisualSceneSTEPGeometry ();
-
-    virtual void BuildScene (int zoomall = 0);
-    virtual void DrawScene ();
-  };
-#endif
-
-
-  class VisualSceneSTLMeshing : public VisualScene
-  {
-    Array<int> trilists;
-    int selecttrig, nodeofseltrig;
-
-  public:
-    VisualSceneSTLMeshing ();
-    virtual ~VisualSceneSTLMeshing ();
-
-    virtual void BuildScene (int zoomall = 0);
-    virtual void DrawScene ();
-    virtual void MouseDblClick (int px, int py);
-
-    int seltria;
-  };
-
-
-
 
   class VisualSceneSurfaceMeshing : public VisualScene
   {
@@ -248,6 +158,7 @@ namespace netgen
     Array<int> par_filledlists;
 #endif
 
+    MouseEventHandler * user_me_handler;
 
     NgLock *lock;
 
@@ -265,6 +176,10 @@ namespace netgen
     virtual void DrawScene ();
     virtual void MouseDblClick (int px, int py);
 
+    void SetMouseEventHandler (MouseEventHandler * handler)
+    { user_me_handler = handler; }
+
+
     int SelectedFace () const
     { return selface; }
     void SetSelectedFace (int asf);
@@ -276,7 +191,7 @@ namespace netgen
     { return selelement; }
     int SelectedPoint () const
     { return selpoint; }
-    void BuildFilledList();
+    void BuildFilledList (bool names);
     // private:
     void BuildLineList();
     void BuildEdgeList();
@@ -294,10 +209,6 @@ namespace netgen
 
 
 
-
-
-
-
   class VisualSceneSpecPoints : public VisualScene
   {
   public:
@@ -310,6 +221,12 @@ namespace netgen
     double len;
   };
 
+
+
+
+
+
+
   // extern struct Tcl_Interp * hinterp;
 
 
