@@ -43,7 +43,9 @@ class VisualSceneSolution : public VisualScene
 
   int surfellist;
   int linelist;
-  int clipplanelist;
+  int element1dlist;
+  int clipplanelist_scal;
+  int clipplanelist_vec;
   int isolinelist;
   int clipplane_isolinelist;
   int surface_vector_list;
@@ -99,7 +101,7 @@ public:
     FUNC_MISES = 3, 
     FUNC_MAIN = 4
   };
-  EvalFunc evalfunc;
+  int evalfunc;
 
   enum SolType
     { 
@@ -138,8 +140,6 @@ public:
 
 
   Array<SolData*> soldata;
-  
-
 
 
   int usetexture;    // 0..no, 1..1D texture (standard), 2..2D-texture (complex)
@@ -191,9 +191,13 @@ public:
 
   void SaveSolutionData (const char * filename);
 
-
+  /*
   static void RealVec3d (const double * values, Vec3d & v, 
 			 bool iscomplex, bool imag);
+  */
+  static Vec<3> RealVec3d (const double * values, 
+			   bool iscomplex, bool imag);
+
   static void RealVec3d (const double * values, Vec3d & v, 
 			 bool iscomplex, double phaser, double phasei);
 
@@ -204,9 +208,9 @@ public:
     subdivision_timestamp = solutiontimestamp = NextTimeStamp();
   }
 
-private:
   void GetMinMax (int funcnr, int comp, double & minv, double & maxv) const;
 
+private:
   void GetClippingPlaneTrigs (Array<ClipPlaneTrig> & trigs, Array<ClipPlanePoint> & pts);
   void GetClippingPlaneGrid (Array<ClipPlanePoint> & pts);
   void DrawCone (const Point<3> & p1, const Point<3> & p2, double r);
@@ -234,35 +238,36 @@ private:
 		  const double xref[], const double x[], const double dxdxref[], 
 		  double * values) const;
 
-  bool GetMultiValues (const SolData * data, ElementIndex elnr, int npt,
+  bool GetMultiValues (const SolData * data, ElementIndex elnr, int facetnr, int npt,
 		       const double * xref, int sxref,
 		       const double * x, int sx,
 		       const double * dxdxref, int sdxdxref,
 		       double * val, int sval) const;
 
 
-  bool GetSurfValue (const SolData * data, SurfaceElementIndex elnr,
+  bool GetSurfValue (const SolData * data, SurfaceElementIndex elnr, int facetnr,
 		     double lam1, double lam2, 
 		     int comp, double & val) const;
 
-  bool GetSurfValue (const SolData * data, SurfaceElementIndex elnr,
+  bool GetSurfValue (const SolData * data, SurfaceElementIndex elnr, int facetnr, 
 		     const double xref[], const double x[], const double dxdxref[], 
 		     int comp, double & val) const;
 
   
-  bool GetSurfValueComplex (const SolData * data, SurfaceElementIndex elnr,
+  bool GetSurfValueComplex (const SolData * data, SurfaceElementIndex elnr, int facetnr, 
 			    double lam1, double lam2, 
 			    int comp, complex<double> & val) const;
 
-  bool GetSurfValues (const SolData * data, SurfaceElementIndex elnr,
+  bool GetSurfValues (const SolData * data, SurfaceElementIndex elnr, int facetnr, 
 		      double lam1, double lam2, 
 		      double * values) const;
 
-  bool GetSurfValues (const SolData * data, SurfaceElementIndex elnr,
+  bool GetSurfValues (const SolData * data, SurfaceElementIndex elnr, int facetnr, 
 		      const double xref[], const double x[], const double dxdxref[], 
 		      double * values) const;
 
-  bool GetMultiSurfValues (const SolData * data, SurfaceElementIndex elnr, int npt,
+  bool GetMultiSurfValues (const SolData * data, SurfaceElementIndex elnr, int facetnr, 
+			   int npt,
                            const double * xref, int sxref,
                            const double * x, int sx,
                            const double * dxdxref, int sdxdxref,
@@ -273,7 +278,7 @@ private:
 
 
   Vec<3> GetDeformation (ElementIndex elnr, const Point<3> & p) const;
-  Vec<3> GetSurfDeformation (SurfaceElementIndex selnr, double lam1, double lam2) const;
+  Vec<3> GetSurfDeformation (SurfaceElementIndex selnr, int facetnr, double lam1, double lam2) const;
 
   void GetPointDeformation (int pnum, Point<3> & p, SurfaceElementIndex elnr = -1) const;
 
@@ -281,6 +286,8 @@ public:
   /// draw elements (build lists)
   void DrawSurfaceElements ();
   void DrawSurfaceElementLines ();
+  void Draw1DElements();
+
   void DrawSurfaceVectors ();
   void DrawTrigSurfaceVectors(const Array< Point<3> > & lp, const Point<3> & pmin, const Point<3> & pmax,
 			      const int sei, const SolData * vsol);
@@ -290,7 +297,6 @@ public:
 		     const Point<3> & p2, 
 		     const Point<3> & p3,
 		     double val1, double val2, double val3);
-  // double minval, double maxval, int n);
 
   // draw isolines between lines (p1,p2) and (p3,p4)
   void DrawIsoLines2 (const Point<3> & p1, 
@@ -299,15 +305,9 @@ public:
 		      const Point<3> & p4,
 		      double val1, double val2, double val3, double val4);
 
-  // double minval, double maxval, int n);
-		     
-
-  void DrawClipPlaneTrigs (const SolData * sol, int comp,
-                           const Array<ClipPlaneTrig> & trigs, 
-                           const Array<ClipPlanePoint> & points);
-			  
-  // void SetOpenGlColor(double val, double valmin, double valmax, int logscale = 0);
 
+  void DrawClipPlaneTrigs (); // const SolData * sol, int comp);
+		  
   void SetOpenGlColor(double val);
 
   // 0 .. non, 1 .. scalar, 2 .. complex
