@@ -5,16 +5,14 @@ set oldmousex 0
 set oldmousey 0
 #
 
+
 # if { 1 } {
 
 # use this one for Togl 2.0
+# if {[catch {togl .ndraw -width 400 -height 300  -rgba true -double true -depth true -privatecmap false -stereo false -indirect true -create init  -display draw -reshape reshape  }] } {    
 
-# if {[catch {togl .ndraw -width 400 -height 300  -rgba true -double true -depth true -privatecmap false -stereo false -indirect true -create init  -display draw -reshape reshape  }] } {
 
-# changed -indirect true/false !!!
-if {[catch {togl .ndraw -width 400 -height 300  -rgba true -double true -depth true -privatecmap false -stereo false -indirect true }] } {    
-
-    puts "no OpenGL" 
+if {[catch {togl .ndraw -width 400 -height 300  -rgba true -double true -depth true -privatecmap false -stereo false -indirect true }] } {    puts "no OpenGL" 
 } {
     #
     pack .ndraw -expand true -fill both -padx 10 -pady 10
@@ -72,6 +70,7 @@ proc popupcheckredraw2 { vari boolvar { x 0 } } {
 	set varname 0
     } {
 	Ng_SetVisParameters
+	Ng_Vis_Set parameters
 	if { $boolvar == 1 } { redraw }
 	Ng_SetVisParameters
     }
