@@ -42,12 +42,12 @@ menu .ngmenu.file
 	    lappend types {"ACIS Geometry" {.sat} }
 	}
 
-	
 	if {[catch {
 	    set file [tk_getOpenFile -filetypes $types -initialdir $dirname -typevariable loadgeomtypevar]
 	}]} {
 	    set file [tk_getOpenFile -filetypes $types -initialdir $dirname]
 	}
+
 	if {$file != ""} {
 	    AddRecentFile $file
 	    Ng_LoadGeometry $file 
@@ -62,7 +62,9 @@ menu .ngmenu.file
 	    set dirname [file dirname $file]
 	    set basefilename [file tail [file rootname $file]]
 
-	    rebuildoccdialog
+	    if { $hasocc == "yes" } {
+		rebuildoccdialog
+	    }
 	}
     }
 
@@ -118,7 +120,8 @@ proc AddRecentFile { filename } {
 		  wm title . [concat \" $progname - $filename \"];
                   set dirname {[file dirname $filename]};
                   set basefilename {[file tail [file rootname $filename]]};
-                  rebuildoccdialog;"
+        	  rebuildoccdialog;"
+
     
     if { [.ngmenu.file.recent index last] >= 6 } {
 	.ngmenu.file.recent delete last }
@@ -133,7 +136,7 @@ loadinifile;
 .ngmenu.file add command -label "Load Mesh..." -accelerator "<l><m>" \
     -command {
 	set types {
-	    {"Mesh file"   {.vol}	} }
+	    {"Mesh file"   {.vol .vol.gz}	} }
 	set file [tk_getOpenFile -filetypes $types -defaultextension ".vol"]
 	if {$file != ""} {
 	    AddRecentMeshFile $file;
@@ -184,9 +187,9 @@ loadmeshinifile;
 .ngmenu.file add command -label "Save Mesh..." -accelerator "<s><m>" \
     -command {
 	set types {
-	    {"Mesh file"   {.vol}	} }
+	    {"Mesh file"   {.vol .vol.gz}	} }
 
-	set file [tk_getSaveFile -filetypes $types -defaultextension ".vol" -initialfile $basefilename -initialdir $dirname ]
+	set file [tk_getSaveFile -filetypes $types -defaultextension ".vol.gz" -initialfile $basefilename -initialdir $dirname ]
 	if {$file != ""} {
 	    Ng_SaveMesh $file }
 	AddRecentMeshFile $file;
@@ -246,6 +249,8 @@ loadmeshinifile;
 	    set file [file nativename [tk_chooseDirectory -title "Elmer Mesh Export - Select Directory"]]
         } elseif { $exportfiletype == "OpenFOAM 1.5+ Format"} {
 	    set file [file nativename [tk_chooseDirectory -title "OpenFOAM 1.5+ Mesh Export - Select Case Directory"]]
+        } elseif { $exportfiletype == "OpenFOAM 1.5+ Compressed"} {
+	    set file [file nativename [tk_chooseDirectory -title "OpenFOAM 1.5+ Mesh Export - Select Case Directory"]]
         } else {
 #	    set file [tk_getSaveFile  -filetypes "{ \"$exportfiletype\" {$extension} }" ]
 	    set file [tk_getSaveFile  -filetypes "{ \"$exportfiletype\" {*}}" ]
@@ -399,7 +404,17 @@ set videoactive 0
 # .ngmenu.file add separator
 
 .ngmenu.file add command -label "Quit" -accelerator "<q>" \
-    -command { puts "Thank you for using $progname"; Ng_Exit; destroy . }
+    -command { 
+        puts "Thank you for using $progname"; 
+
+        if { [catch { unload libngsolve[info sharedlibextension] ngsolve } result ] } {
+#            puts "cannot unload ngsolve" 
+#            puts "error: $result"
+        } 
+
+        Ng_Exit; 
+        destroy . 
+    }
 # exit
 
 
@@ -412,10 +427,10 @@ set videoactive 0
 menu .ngmenu.mesh
 .ngmenu.mesh add command -label "Generate Mesh" -accelerator "<g><m>" \
     -command { 
-	Ng_GenerateMesh ${meshoptions.firststep} ${meshoptions.laststep}
-	Ng_ReadStatus
 	set selectvisual mesh
 	Ng_SetVisParameters
+	Ng_GenerateMesh ${meshoptions.firststep} ${meshoptions.laststep}
+	Ng_ReadStatus
 	redraw
     }
 
@@ -526,67 +541,8 @@ menu .ngmenu.mesh.surfoptstep
 #####################################################
 
 menu .ngmenu.geometry
-.ngmenu.geometry add command -label "Scan CSG Geometry" -command { Ng_ParseGeometry }
-.ngmenu.geometry add command -label "CSG Options..." -command geometryoptionsdialog
-# only intern version !
-# .ngmenu.geometry add separator
-# .ngmenu.geometry add command -label "New Primitive"  \
-#     -command newprimitivedialog -accelerator "<n><p>"
-# .ngmenu.geometry add command -label "Edit Primitive" \
-#     -command editprimitivedialog -accelerator "<e><p>"
-# .ngmenu.geometry add command -label "Edit Solid" \
-#     -command newsoliddialog -accelerator "<e><s>"
-# .ngmenu.geometry add command -label "Choose Top Level " \
-#     -command topleveldialog 
-# .ngmenu.geometry add command -label "Identify" \
-#     -command identifydialog 
-.ngmenu.geometry add command -label "CSG Properties..." \
-    -command topleveldialog2 
 
-.ngmenu.geometry add separator
 
-.ngmenu.geometry add command -label "STL Doctor..." \
-    -command { stldoctordialog; }
-
-.ngmenu.geometry add command -label "STL Info" \
-    -command {
-	set notriangles 0
-	set minx 0
-	set maxx 0
-	set miny 0
-	set maxy 0
-	set minz 0
-	set maxz 0
-	set trigscons 0
-	Ng_STLInfo notriangles minx maxx miny maxy minz maxz trigscons
-	set msgtext "NO STL-Triangles : $notriangles\nGeometry:\nX = $minx - $maxx\nY = $miny - $maxy\nZ = $minz - $maxz\nConsistency Check = $trigscons\n"
-	set msgtext "$msgtext Status: [Ng_STLInfo status]"
-	tk_messageBox -title "STL Info" -message  $msgtext -type ok 
-    }
-
-.ngmenu.geometry add separator
-
-.ngmenu.geometry add command -label "IGES/STEP Topology Explorer/Doctor..." \
-    -command { occdialog; }
-
-# Philippose - 30/01/2009
-# Add menu item for local face mesh size definition in the 
-# TCL Gui
-.ngmenu.geometry add command -label "Edit Face Mesh Size..." \
-    -command { surfacemeshsizedialog }
-
-.ngmenu.geometry add command -label "OCC Construction" \
-    -command { Ng_OCCConstruction; }
-
-if { [Ng_ACISCommand isACISavailable] == "yes" } {
-    .ngmenu.geometry add command -label "ACIS Topology Explorer..." \
-        -command { acisdialog; }
-
-    .ngmenu.geometry add command -label "ACIS combine all" \
-        -command { Ng_ACISCommand combineall }
-    .ngmenu.geometry add command -label "ACIS Create CT" \
-        -command { Ng_ACISCommand createct }
-}
 
 
 
@@ -788,6 +744,11 @@ menu .ngmenu.special
 .ngmenu.special add cascade -label "Colour-based boundary conditions" -menu .ngmenu.special.colbndcond
 
 menu .ngmenu.special.colbndcond 
+ .ngmenu.special.colbndcond add command -label "Inspect Colours in mesh" \
+    -command { currmeshcoloursdialog }
+    
+ .ngmenu.special.colbndcond add separator	
+    
  .ngmenu.special.colbndcond add command -label "Automatic Assignment" \
     -command { Ng_AutoColourBcProps auto; redraw }
 	
@@ -859,13 +820,19 @@ pack .bubar -side top -fill x
 
 button .bubar.testb -text "Test" -command { Ng_SaveGeometry }
 button .bubar.surfm -text "Generate Mesh" -command \
-    { set selectvisual mesh; 
-	Ng_SetVisParameters;
-	Ng_GenerateMesh ${meshoptions.firststep} ${meshoptions.laststep}
-	redraw 
+    { 
+	.ngmenu.mesh invoke "Generate Mesh"; 
+#	set selectvisual mesh; 
+#	Ng_SetVisParameters;
+#	Ng_GenerateMesh ${meshoptions.firststep} ${meshoptions.laststep}
+#	redraw 
     }
 button .bubar.stopm -text "Stop" -command \
-    { Ng_StopMeshing;  set stopdemo 1 }
+    { 
+	# Ng_StopMeshing;  
+	set multithread_terminate 1;
+	set stopdemo 1;
+    }
 button .bubar.exitb -text "Quit" \
     -command { 
 	   set ans [tk_messageBox -title "Quit Netgen?" -message "Do you really want to quit Netgen?" -type yesno -default "no" -icon question]
@@ -878,17 +845,6 @@ pack  .bubar.exitb .bubar.surfm .bubar.stopm -side left
 #button .bubar.scan -text "Scan" \
 #    -command { Ng_ParseGeometry; set selectvisual geometry; Ng_SetVisParameters; redraw }
 
-# fuer parallel - buttons :)
-Ng_IsParallel;
-if { $parallel_netgen } {
-#    catch{ 
-#    source ${ngdir}/ngtcltk/parallel_dialog.tcl
-#    }
-    button .bubar.visallb -text "Parallel" -command \
-	{ paralleldialog; redraw } 
-    pack .bubar.visallb -side left 
-}
-
 button .bubar.zoomall -text "Zoom All" \
     -command { Ng_ZoomAll; redraw }
 
