@@ -40,7 +40,7 @@
 #define NG_VOLUME_ELEMENT_MAXPOINTS 10
 
 /// Maximum allowed number of nodes per surface element
-#define NG_SURFACE_ELEMENT_MAXPOINTS 6
+#define NG_SURFACE_ELEMENT_MAXPOINTS 8
 
 
 
@@ -67,7 +67,7 @@ typedef void * Ng_OCC_TopTools_IndexedMapOfShape;
 // *** Special Enum types used within Netgen ***********
 /// Currently implemented surface element types
 enum Ng_Surface_Element_Type 
-   { NG_TRIG = 1, NG_QUAD = 2, NG_TRIG6 = 3 };
+   { NG_TRIG = 1, NG_QUAD = 2, NG_TRIG6 = 3, NG_QUAD6 = 4, NG_QUAD8 = 5 };
 
 /// Currently implemented volume element types
 enum Ng_Volume_Element_Type 
@@ -92,30 +92,39 @@ enum Ng_Result
 class Ng_Meshing_Parameters 
 {
 public:
-   int uselocalh;              //!< Switch to enable / disable usage of local mesh size modifiers
+   int uselocalh;                      //!< Switch to enable / disable usage of local mesh size modifiers
 
-   double maxh;                //!< Maximum global mesh size allowed
-   double minh;                //!< Minimum global mesh size allowed
+   double maxh;                        //!< Maximum global mesh size allowed
+   double minh;                        //!< Minimum global mesh size allowed
 
-   double fineness;            //!< Mesh density: 0...1 (0 => coarse; 1 => fine)
-   double grading;             //!< Mesh grading: 0...1 (0 => uniform mesh; 1 => aggressive local grading)
+   double fineness;                    //!< Mesh density: 0...1 (0 => coarse; 1 => fine)
+   double grading;                     //!< Mesh grading: 0...1 (0 => uniform mesh; 1 => aggressive local grading)
 
-   double elementsperedge;     //!< Number of elements to generate per edge of the geometry
-   double elementspercurve;    //!< Elements to generate per curvature radius
+   double elementsperedge;             //!< Number of elements to generate per edge of the geometry
+   double elementspercurve;            //!< Elements to generate per curvature radius
 
-   int closeedgeenable;        //!< Enable / Disable mesh refinement at close edges
-   double closeedgefact;       //!< Factor to use for refinement at close edges (STL: larger => finer ; OCC: larger => coarser)
+   int closeedgeenable;                //!< Enable / Disable mesh refinement at close edges
+   double closeedgefact;               //!< Factor to use for refinement at close edges (larger => finer)
 
-   int secondorder;            //!< Generate second-order surface and volume elements
-   int quad_dominated;         //!< Creates a Quad-dominated mesh 
+   int second_order;                   //!< Generate second-order surface and volume elements
+   int quad_dominated;                 //!< Creates a Quad-dominated mesh 
 
-   char * meshsize_filename;   //!< Optional external mesh size file 
+   char * meshsize_filename;           //!< Optional external mesh size file 
 
-   int optsurfmeshenable;      //!< Enable / Disable automatic surface mesh optimization
-   int optvolmeshenable;       //!< Enable / Disable automatic volume mesh optimization
+   int optsurfmeshenable;              //!< Enable / Disable automatic surface mesh optimization
+   int optvolmeshenable;               //!< Enable / Disable automatic volume mesh optimization
 
-   int optsteps_3d;            //!< Number of optimize steps to use for 3-D mesh optimization
-   int optsteps_2d;            //!< Number of optimize steps to use for 2-D mesh optimization
+   int optsteps_3d;                     //!< Number of optimize steps to use for 3-D mesh optimization
+   int optsteps_2d;                     //!< Number of optimize steps to use for 2-D mesh optimization
+
+   // Philippose - 13/09/2010
+   // Added a couple more parameters into the meshing parameters list 
+   // from Netgen into Nglib
+   int invert_tets;                    //!< Invert all the volume elements
+   int invert_trigs;                   //!< Invert all the surface triangle elements
+
+   int check_overlap;                  //!< Check for overlapping surfaces during Surface meshing
+   int check_overlapping_boundary;     //!< Check for overlapping surface elements before volume meshing
 
 
    /*!
@@ -131,13 +140,17 @@ public:
       - #elementspercurve: 2.0
       - #closeedgeenable: 0
       - #closeedgefact: 2.0
-      - #secondorder: 0.0
+      - #secondorder: 0
       - #meshsize_filename: null
       - #quad_dominated: 0
       - #optsurfmeshenable: 1
       - #optvolmeshenable: 1
       - #optsteps_2d: 3
       - #optsteps_3d: 3
+      - #invert_tets: 0
+      - #invert_trigs:0 
+      - #check_overlap: 1
+      - #check_overlapping_boundary: 1
    */
    DLL_HEADER Ng_Meshing_Parameters();
 
@@ -150,6 +163,17 @@ public:
        of the object to the default values
    */
    DLL_HEADER void Reset_Parameters();
+
+
+
+   /*!
+       Transfer local meshing parameters to internal meshing parameters
+
+       This member function transfers all the meshing parameters 
+       defined in the local meshing parameters structure of nglib into 
+       the internal meshing parameters structure used by the Netgen core
+   */
+   DLL_HEADER void Transfer_Parameters();
 };
 
 
@@ -230,6 +254,35 @@ DLL_HEADER void Ng_SaveMesh(Ng_Mesh * mesh, const char* filename);
 */
 DLL_HEADER Ng_Mesh * Ng_LoadMesh(const char* filename);
 
+
+/*! \brief Merge a Netgen VOL Mesh from disk into an existing mesh in memory
+
+    A Netgen mesh saved in the internal VOL format can be merged 
+    into an existing Netgen Mesh structure using this function. 
+
+    \param mesh       Name of the Mesh structure already existent in memory
+    \param filename   Pointer to a character array containing the 
+                      name of the file to load
+    \return Ng_Result Status of the merge operation
+*/
+DLL_HEADER Ng_Result Ng_MergeMesh(Ng_Mesh * mesh, const char* filename);
+
+
+/*! \brief Merge one Netgen Mesh into another Netgen Mesh in the case 
+    when both are already in memory
+
+    (NOTE: FUNCTION STILL WORK IN PROGRESS!!!)
+
+    This function can be used to merge two Netgen meshes already present 
+    in memory.
+
+    \param mesh1      Parent Mesh structure into which the second mesh 
+                      will be merged
+    \param mesh2      Child mesh structure which will get merged into 
+                      the parent mesh
+    \return Ng_Result Status of the merge operation
+*/
+DLL_HEADER Ng_Result Ng_MergeMesh(Ng_Mesh * mesh1, Ng_Mesh * mesh2);
 // ------------------------------------------------------------------
 
 
@@ -508,8 +561,9 @@ DLL_HEADER int Ng_GetNSeg_2D (Ng_Mesh * mesh);
 //  return point coordinates
 DLL_HEADER void Ng_GetPoint_2D (Ng_Mesh * mesh, int num, double * x);
 
-// return 2d triangles
-DLL_HEADER void Ng_GetElement_2D (Ng_Mesh * mesh, int num, int * pi, int * matnum = NULL);
+// return 2d elements
+DLL_HEADER Ng_Surface_Element_Type 
+Ng_GetElement_2D (Ng_Mesh * mesh, int num, int * pi, int * matnum = NULL);
 
 // return 2d boundary segment
 DLL_HEADER void Ng_GetSegment_2D (Ng_Mesh * mesh, int num, int * pi, int * matnum = NULL);
@@ -636,14 +690,10 @@ DLL_HEADER Ng_Result Ng_OCC_GetFMap(Ng_OCC_Geometry * geom,
 
 
 
-
-
-
 // **********************************************************
 // **   Mesh refinement algorithms                         **
 // **********************************************************
 
-
 // uniform mesh refinement
 DLL_HEADER void Ng_Uniform_Refinement (Ng_Mesh * mesh);
 
@@ -666,4 +716,29 @@ DLL_HEADER void Ng_OCC_Uniform_Refinement (Ng_OCC_Geometry * geom,
 
 
 
+// **********************************************************
+// **   Second Order mesh algorithms                       **
+// **********************************************************
+
+// convert mesh to second order
+DLL_HEADER void Ng_Generate_SecondOrder (Ng_Mesh * mesh);
+
+
+// convert mesh to second order with geometry adaption:
+
+DLL_HEADER void Ng_2D_Generate_SecondOrder (Ng_Geometry_2D * geom,
+					  Ng_Mesh * mesh);
+
+DLL_HEADER void Ng_STL_Generate_SecondOrder (Ng_STL_Geometry * geom,
+					   Ng_Mesh * mesh);
+
+DLL_HEADER void Ng_CSG_Generate_SecondOrder (Ng_CSG_Geometry * geom,
+					   Ng_Mesh * mesh);
+
+#ifdef OCCGEOMETRY
+DLL_HEADER void Ng_OCC_Generate_SecondOrder (Ng_OCC_Geometry * geom,
+					   Ng_Mesh * mesh);
+#endif
+
+
 #endif // NGLIB
